@@ -30,8 +30,10 @@ void ags_property_collection_editor_class_init(AgsPropertyCollectionEditorClass
 void ags_property_collection_editor_connectable_interface_init(AgsConnectableInterface *connectable);
 void ags_property_collection_editor_applicable_interface_init(AgsApplicableInterface *applicable);
 void ags_property_collection_editor_init(AgsPropertyCollectionEditor *property_collection_editor);
+
 void ags_property_collection_editor_connect(AgsConnectable *connectable);
 void ags_property_collection_editor_disconnect(AgsConnectable *connectable);
+
 void ags_property_collection_editor_set_update(AgsApplicable *applicable, gboolean update);
 void ags_property_collection_editor_apply(AgsApplicable *applicable);
 void ags_property_collection_editor_reset(AgsApplicable *applicable);
@@ -55,7 +57,7 @@ ags_property_collection_editor_get_type(void)
   static volatile gsize g_define_type_id__volatile = 0;
 
   if(g_once_init_enter (&g_define_type_id__volatile)){
-    GType ags_type_property_collection_editor;
+    GType ags_type_property_collection_editor = 0;
 
     static const GTypeInfo ags_property_collection_editor_info = {
       sizeof (AgsPropertyCollectionEditorClass),
@@ -93,7 +95,7 @@ ags_property_collection_editor_get_type(void)
 				AGS_TYPE_APPLICABLE,
 				&ags_applicable_interface_info);
 
-    g_once_init_leave (&g_define_type_id__volatile, ags_type_property_collection_editor);
+    g_once_init_leave(&g_define_type_id__volatile, ags_type_property_collection_editor);
   }
 
   return g_define_type_id__volatile;
@@ -234,7 +236,7 @@ ags_property_collection_editor_reset(AgsApplicable *applicable)
  *
  * Returns: a new #AgsPropertyCollectionEditor
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 AgsPropertyCollectionEditor*
 ags_property_collection_editor_new(GType child_type,
