@@ -62,7 +62,7 @@
 #endif
 
 #ifdef AGS_WITH_QUARTZ
-#include <gtkmacintegration/gtkosxapplication.h>
+#include <gtkmacintegration-gtk2/gtkosxapplication.h>
 #endif
 
 #include <sys/types.h>
@@ -108,7 +108,11 @@ void ags_xorg_application_context_set_default_soundcard_thread(AgsSoundProvider
 GList* ags_xorg_application_context_get_sequencer(AgsSoundProvider *sound_provider);
 void ags_xorg_application_context_set_sequencer(AgsSoundProvider *sound_provider,
 						GList *sequencer);
-GList* ags_xorg_application_context_get_distributed_manager(AgsSoundProvider *sound_provider);
+GList* ags_xorg_application_context_get_sound_server(AgsSoundProvider *sound_provider);
+GList* ags_xorg_application_context_get_audio(AgsSoundProvider *sound_provider);
+void ags_xorg_application_context_set_audio(AgsSoundProvider *sound_provider,
+					    GList *soundcard);
+
 GtkWidget* ags_xorg_application_context_get_window(AgsUiProvider *ui_provider);
 void ags_xorg_application_context_set_window(AgsUiProvider *ui_provider,
 					     GtkWidget *widget);
@@ -181,9 +185,11 @@ struct itimerspec its;
 GType
 ags_xorg_application_context_get_type()
 {
-  static GType ags_type_xorg_application_context = 0;
+  static volatile gsize g_define_type_id__volatile = 0;
+
+  if(g_once_init_enter (&g_define_type_id__volatile)){
+    GType ags_type_xorg_application_context = 0;
 
-  if(!ags_type_xorg_application_context){
     static const GTypeInfo ags_xorg_application_context_info = {
       sizeof (AgsXorgApplicationContextClass),
       NULL, /* base_init */
@@ -240,9 +246,11 @@ ags_xorg_application_context_get_type()
     g_type_add_interface_static(ags_type_xorg_application_context,
 				AGS_TYPE_UI_PROVIDER,
 				&ags_ui_provider_interface_info);
+
+    g_once_init_leave(&g_define_type_id__volatile, ags_type_xorg_application_context);
   }
 
-  return (ags_type_xorg_application_context);
+  return g_define_type_id__volatile;
 }
 
 #ifndef AGS_USE_TIMER
@@ -311,7 +319,7 @@ ags_xorg_application_context_class_init(AgsXorgApplicationContextClass *xorg_app
    *
    * The assigned window.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_object("window",
 				   i18n_pspec("window of xorg application context"),
@@ -362,11 +370,17 @@ ags_xorg_application_context_sound_provider_interface_init(AgsSoundProviderInter
 {
   sound_provider->get_soundcard = ags_xorg_application_context_get_soundcard;
   sound_provider->set_soundcard = ags_xorg_application_context_set_soundcard;
+
   sound_provider->get_default_soundcard_thread = ags_xorg_application_context_get_default_soundcard_thread;
   sound_provider->set_default_soundcard_thread = ags_xorg_application_context_set_default_soundcard_thread;
+
   sound_provider->get_sequencer = ags_xorg_application_context_get_sequencer;
   sound_provider->set_sequencer = ags_xorg_application_context_set_sequencer;
-  sound_provider->get_distributed_manager = ags_xorg_application_context_get_distributed_manager;
+
+  sound_provider->get_sound_server = ags_xorg_application_context_get_sound_server;
+
+  sound_provider->get_audio = ags_xorg_application_context_get_audio;
+  sound_provider->set_audio = ags_xorg_application_context_set_audio;
 }
 
 void
@@ -433,8 +447,10 @@ ags_xorg_application_context_init(AgsXorgApplicationContext *xorg_application_co
   xorg_application_context->soundcard = NULL;
   xorg_application_context->sequencer = NULL;
 
-  xorg_application_context->distributed_manager = NULL;
+  xorg_application_context->sound_server = NULL;
   
+  xorg_application_context->audio = NULL;
+
   xorg_application_context->window = NULL;
 }
 
@@ -582,14 +598,50 @@ ags_xorg_application_context_set_worker(AgsConcurrencyProvider *concurrency_prov
 GList*
 ags_xorg_application_context_get_soundcard(AgsSoundProvider *sound_provider)
 {
-  return(AGS_XORG_APPLICATION_CONTEXT(sound_provider)->soundcard);
+  GList *soundcard;
+
+  pthread_mutex_t *application_context_mutex;
+
+  /* get mutex */
+  pthread_mutex_lock(ags_application_context_get_class_mutex());
+  
+  application_context_mutex = AGS_APPLICATION_CONTEXT(sound_provider)->obj_mutex;
+
+  pthread_mutex_unlock(ags_application_context_get_class_mutex());
+
+  /* get soundcard */
+  pthread_mutex_lock(application_context_mutex);
+  
+  soundcard = g_list_copy(AGS_XORG_APPLICATION_CONTEXT(sound_provider)->soundcard);
+
+  pthread_mutex_unlock(application_context_mutex);
+  
+  return(soundcard);
 }
 
 void
 ags_xorg_application_context_set_soundcard(AgsSoundProvider *sound_provider,
 					   GList *soundcard)
 {
+  pthread_mutex_t *application_context_mutex;
+
+  /* get mutex */
+  pthread_mutex_lock(ags_application_context_get_class_mutex());
+  
+  application_context_mutex = AGS_APPLICATION_CONTEXT(sound_provider)->obj_mutex;
+
+  pthread_mutex_unlock(ags_application_context_get_class_mutex());
+
+  /* set soundcard */
+  pthread_mutex_lock(application_context_mutex);
+
+  if(AGS_XORG_APPLICATION_CONTEXT(sound_provider)->soundcard != NULL){
+    g_list_free(AGS_XORG_APPLICATION_CONTEXT(sound_provider)->soundcard);
+  }
+
   AGS_XORG_APPLICATION_CONTEXT(sound_provider)->soundcard = soundcard;
+
+  pthread_mutex_unlock(application_context_mutex);
 }
 
 GObject*
@@ -608,20 +660,123 @@ ags_xorg_application_context_set_default_soundcard_thread(AgsSoundProvider *soun
 GList*
 ags_xorg_application_context_get_sequencer(AgsSoundProvider *sound_provider)
 {
-  return(AGS_XORG_APPLICATION_CONTEXT(sound_provider)->sequencer);
+  GList *sequencer;
+
+  pthread_mutex_t *application_context_mutex;
+
+  /* get mutex */
+  pthread_mutex_lock(ags_application_context_get_class_mutex());
+  
+  application_context_mutex = AGS_APPLICATION_CONTEXT(sound_provider)->obj_mutex;
+
+  pthread_mutex_unlock(ags_application_context_get_class_mutex());
+
+  /* get sequencer */
+  pthread_mutex_lock(application_context_mutex);
+  
+  sequencer = g_list_copy(AGS_XORG_APPLICATION_CONTEXT(sound_provider)->sequencer);
+
+  pthread_mutex_unlock(application_context_mutex);
+  
+  return(sequencer);
 }
 
 void
 ags_xorg_application_context_set_sequencer(AgsSoundProvider *sound_provider,
 					   GList *sequencer)
 {
+  pthread_mutex_t *application_context_mutex;
+
+  /* get mutex */
+  pthread_mutex_lock(ags_application_context_get_class_mutex());
+  
+  application_context_mutex = AGS_APPLICATION_CONTEXT(sound_provider)->obj_mutex;
+
+  pthread_mutex_unlock(ags_application_context_get_class_mutex());
+
+  /* set sequencer */
+  pthread_mutex_lock(application_context_mutex);
+
+  if(AGS_XORG_APPLICATION_CONTEXT(sound_provider)->sequencer != NULL){
+    g_list_free(AGS_XORG_APPLICATION_CONTEXT(sound_provider)->sequencer);
+  }
+  
   AGS_XORG_APPLICATION_CONTEXT(sound_provider)->sequencer = sequencer;
+
+  pthread_mutex_unlock(application_context_mutex);
 }
 
 GList*
-ags_xorg_application_context_get_distributed_manager(AgsSoundProvider *sound_provider)
+ags_xorg_application_context_get_sound_server(AgsSoundProvider *sound_provider)
 {
-  return(AGS_XORG_APPLICATION_CONTEXT(sound_provider)->distributed_manager);
+  GList *sound_server;
+
+  pthread_mutex_t *application_context_mutex;
+
+  /* get mutex */
+  pthread_mutex_lock(ags_application_context_get_class_mutex());
+  
+  application_context_mutex = AGS_APPLICATION_CONTEXT(sound_provider)->obj_mutex;
+
+  pthread_mutex_unlock(ags_application_context_get_class_mutex());
+
+  /* get sound server */
+  pthread_mutex_lock(application_context_mutex);
+  
+  sound_server = g_list_copy(AGS_XORG_APPLICATION_CONTEXT(sound_provider)->sound_server);
+
+  pthread_mutex_unlock(application_context_mutex);
+  
+  return(sound_server);
+}
+
+GList*
+ags_xorg_application_context_get_audio(AgsSoundProvider *sound_provider)
+{
+  GList *audio;
+
+  pthread_mutex_t *application_context_mutex;
+
+  /* get mutex */
+  pthread_mutex_lock(ags_application_context_get_class_mutex());
+  
+  application_context_mutex = AGS_APPLICATION_CONTEXT(sound_provider)->obj_mutex;
+
+  pthread_mutex_unlock(ags_application_context_get_class_mutex());
+
+  /* get audio */
+  pthread_mutex_lock(application_context_mutex);
+  
+  audio = g_list_copy(AGS_XORG_APPLICATION_CONTEXT(sound_provider)->audio);
+
+  pthread_mutex_unlock(application_context_mutex);
+  
+  return(audio);
+}
+
+void
+ags_xorg_application_context_set_audio(AgsSoundProvider *sound_provider,
+				       GList *audio)
+{
+  pthread_mutex_t *application_context_mutex;
+
+  /* get mutex */
+  pthread_mutex_lock(ags_application_context_get_class_mutex());
+  
+  application_context_mutex = AGS_APPLICATION_CONTEXT(sound_provider)->obj_mutex;
+
+  pthread_mutex_unlock(ags_application_context_get_class_mutex());
+
+  /* set audio */
+  pthread_mutex_lock(application_context_mutex);
+
+  if(AGS_XORG_APPLICATION_CONTEXT(sound_provider)->audio != NULL){
+    g_list_free(AGS_XORG_APPLICATION_CONTEXT(sound_provider)->audio);
+  }
+
+  AGS_XORG_APPLICATION_CONTEXT(sound_provider)->audio = audio;
+
+  pthread_mutex_unlock(application_context_mutex);
 }
 
 void
@@ -723,8 +878,8 @@ ags_xorg_application_context_dispose(GObject *gobject)
   }
 
   /* distributed manager */
-  if(xorg_application_context->distributed_manager != NULL){
-    list = xorg_application_context->distributed_manager;
+  if(xorg_application_context->sound_server != NULL){
+    list = xorg_application_context->sound_server;
 
     while(list != NULL){
       g_object_set(list->data,
@@ -734,10 +889,10 @@ ags_xorg_application_context_dispose(GObject *gobject)
       list = list->next;
     }
 
-    g_list_free_full(xorg_application_context->distributed_manager,
+    g_list_free_full(xorg_application_context->sound_server,
 		     g_object_unref);
 
-    xorg_application_context->distributed_manager = NULL;
+    xorg_application_context->sound_server = NULL;
   }
 
   /* window */
@@ -799,8 +954,8 @@ ags_xorg_application_context_finalize(GObject *gobject)
 		     g_object_unref);
   }
   
-  if(xorg_application_context->distributed_manager != NULL){
-    g_list_free_full(xorg_application_context->distributed_manager,
+  if(xorg_application_context->sound_server != NULL){
+    g_list_free_full(xorg_application_context->sound_server,
 		     g_object_unref);
   }
   
@@ -876,8 +1031,7 @@ void
 ags_xorg_application_context_prepare(AgsApplicationContext *application_context)
 {
   AgsXorgApplicationContext *xorg_application_context;
-  AgsWindow *window;
-  
+
   AgsThread *audio_loop, *polling_thread, *task_thread;
   AgsThread *gui_thread;
   AgsThreadPool *thread_pool;
@@ -886,17 +1040,6 @@ ags_xorg_application_context_prepare(AgsApplicationContext *application_context)
   
   xorg_application_context = (AgsXorgApplicationContext *) application_context;
 
-#ifdef AGS_WITH_QUARTZ
-    g_object_new(GTKOSX_TYPE_APPLICATION,
-		 NULL);
-#endif
-
-  window = g_object_new(AGS_TYPE_WINDOW,
-			"app-paintable", TRUE,
-			"type", GTK_WINDOW_TOPLEVEL,    
-			"application-context", application_context,
-			NULL);
-  
   /* call parent */
   AGS_APPLICATION_CONTEXT_CLASS(ags_xorg_application_context_parent_class)->prepare(application_context);
   
@@ -987,6 +1130,7 @@ void
 ags_xorg_application_context_setup(AgsApplicationContext *application_context)
 {
   AgsXorgApplicationContext *xorg_application_context;
+  AgsWindow *window;
 
   AgsGuiThread *gui_thread;
   
@@ -1013,7 +1157,6 @@ ags_xorg_application_context_setup(AgsApplicationContext *application_context)
   AgsThread *sequencer_thread;
   AgsDestroyWorker *destroy_worker;
   AgsThread *main_loop;
-  AgsMutexManager *mutex_manager;
 
   AgsLog *log;
   AgsConfig *config;
@@ -1031,7 +1174,8 @@ ags_xorg_application_context_setup(AgsApplicationContext *application_context)
   gchar *soundcard_group;
   gchar *sequencer_group;
   gchar *str;
-      
+  gchar *capability;
+  
   uid_t uid;
   
   guint i;
@@ -1039,8 +1183,7 @@ ags_xorg_application_context_setup(AgsApplicationContext *application_context)
   gboolean has_core_audio;
   gboolean has_pulse;
   gboolean has_jack;
-  
-  pthread_mutex_t *application_mutex;
+  gboolean is_output;
   
   xorg_application_context = (AgsXorgApplicationContext *) application_context;
 
@@ -1051,14 +1194,9 @@ ags_xorg_application_context_setup(AgsApplicationContext *application_context)
   /* call parent */
   AGS_APPLICATION_CONTEXT_CLASS(ags_xorg_application_context_parent_class)->setup(application_context);
 
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
-    
-  pthread_mutex_lock(application_mutex);
-
-  main_loop = AGS_APPLICATION_CONTEXT(xorg_application_context)->main_loop;
-  
-  pthread_mutex_unlock(application_mutex);
+  g_object_get(application_context,
+	       "main-loop", &main_loop,
+	       NULL);
 
   gui_thread = ags_thread_find_type(main_loop,
 				    AGS_TYPE_GUI_THREAD);
@@ -1279,12 +1417,12 @@ ags_xorg_application_context_setup(AgsApplicationContext *application_context)
 #endif
 
   /* distributed manager */
-  xorg_application_context->distributed_manager = NULL;
+  xorg_application_context->sound_server = NULL;
 
   /* core audio server */
   core_audio_server = ags_core_audio_server_new((GObject *) xorg_application_context,
 						NULL);
-  xorg_application_context->distributed_manager = g_list_append(xorg_application_context->distributed_manager,
+  xorg_application_context->sound_server = g_list_append(xorg_application_context->sound_server,
 								core_audio_server);
   g_object_ref(G_OBJECT(core_audio_server));
 
@@ -1293,7 +1431,7 @@ ags_xorg_application_context_setup(AgsApplicationContext *application_context)
   /* pulse server */
   pulse_server = ags_pulse_server_new((GObject *) xorg_application_context,
 				      NULL);
-  xorg_application_context->distributed_manager = g_list_append(xorg_application_context->distributed_manager,
+  xorg_application_context->sound_server = g_list_append(xorg_application_context->sound_server,
 								pulse_server);
   g_object_ref(G_OBJECT(pulse_server));
 
@@ -1302,7 +1440,7 @@ ags_xorg_application_context_setup(AgsApplicationContext *application_context)
   /* jack server */
   jack_server = ags_jack_server_new((GObject *) xorg_application_context,
 				    NULL);
-  xorg_application_context->distributed_manager = g_list_append(xorg_application_context->distributed_manager,
+  xorg_application_context->sound_server = g_list_append(xorg_application_context->sound_server,
 								jack_server);
   g_object_ref(G_OBJECT(jack_server));
 
@@ -1334,42 +1472,71 @@ ags_xorg_application_context_setup(AgsApplicationContext *application_context)
     str = ags_config_get_value(config,
 			       soundcard_group,
 			       "backend");
+
+    capability = ags_config_get_value(config,
+				      soundcard_group,
+				      "capability");
+
+    is_output = TRUE;
+
+    if(capability != NULL &&
+       !g_ascii_strncasecmp(capability,
+			    "capture",
+			    8)){
+      is_output = FALSE;
+    }
     
     /* change soundcard */
     if(str != NULL){
       if(!g_ascii_strncasecmp(str,
 			      "core-audio",
 			      11)){
-	soundcard = ags_distributed_manager_register_soundcard(AGS_DISTRIBUTED_MANAGER(core_audio_server),
-							       TRUE);
+	soundcard = ags_sound_server_register_soundcard(AGS_SOUND_SERVER(core_audio_server),
+							is_output);
 
 	has_core_audio = TRUE;
       }else if(!g_ascii_strncasecmp(str,
 			      "pulse",
 			      6)){
-	soundcard = ags_distributed_manager_register_soundcard(AGS_DISTRIBUTED_MANAGER(pulse_server),
-							       TRUE);
+	soundcard = ags_sound_server_register_soundcard(AGS_SOUND_SERVER(pulse_server),
+							is_output);
 
 	has_pulse = TRUE;
       }else if(!g_ascii_strncasecmp(str,
 			      "jack",
 			      5)){
-	soundcard = ags_distributed_manager_register_soundcard(AGS_DISTRIBUTED_MANAGER(jack_server),
-							       TRUE);
+	soundcard = ags_sound_server_register_soundcard(AGS_SOUND_SERVER(jack_server),
+							is_output);
 
 	has_jack = TRUE;
       }else if(!g_ascii_strncasecmp(str,
 				    "alsa",
 				    5)){
-	soundcard = (GObject *) ags_devout_new((GObject *) xorg_application_context);
-	AGS_DEVOUT(soundcard)->flags &= (~AGS_DEVOUT_OSS);
-	AGS_DEVOUT(soundcard)->flags |= AGS_DEVOUT_ALSA;
+	if(is_output){
+	  soundcard = (GObject *) ags_devout_new((GObject *) xorg_application_context);
+	  
+	  AGS_DEVOUT(soundcard)->flags &= (~AGS_DEVOUT_OSS);
+	  AGS_DEVOUT(soundcard)->flags |= AGS_DEVOUT_ALSA;
+	}else{
+	  soundcard = (GObject *) ags_devin_new((GObject *) xorg_application_context);
+	  
+	  AGS_DEVIN(soundcard)->flags &= (~AGS_DEVIN_OSS);
+	  AGS_DEVIN(soundcard)->flags |= AGS_DEVIN_ALSA;
+	}
       }else if(!g_ascii_strncasecmp(str,
 				    "oss",
 				    4)){
-	soundcard = (GObject *) ags_devout_new((GObject *) xorg_application_context);
-	AGS_DEVOUT(soundcard)->flags &= (~AGS_DEVOUT_ALSA);
-	AGS_DEVOUT(soundcard)->flags |= AGS_DEVOUT_OSS;
+	if(is_output){
+	  soundcard = (GObject *) ags_devout_new((GObject *) xorg_application_context);
+
+	  AGS_DEVOUT(soundcard)->flags &= (~AGS_DEVOUT_ALSA);
+	  AGS_DEVOUT(soundcard)->flags |= AGS_DEVOUT_OSS;
+	}else{
+	  soundcard = (GObject *) ags_devin_new((GObject *) xorg_application_context);
+
+	  AGS_DEVIN(soundcard)->flags &= (~AGS_DEVIN_ALSA);
+	  AGS_DEVIN(soundcard)->flags |= AGS_DEVIN_OSS;	  
+	}
       }else{
 	g_warning(i18n("unknown soundcard backend - %s"), str);
 
@@ -1393,7 +1560,7 @@ ags_xorg_application_context_setup(AgsApplicationContext *application_context)
 
     if(xorg_application_context->soundcard == NULL){
       g_object_set(audio_loop,
-		   "soundcard", G_OBJECT(soundcard),
+		   "default-output-soundcard", G_OBJECT(soundcard),
 		   NULL);
     }
     
@@ -1511,16 +1678,9 @@ ags_xorg_application_context_setup(AgsApplicationContext *application_context)
     /* change sequencer */
     if(str != NULL){
       if(!g_ascii_strncasecmp(str,
-			      "core-audio",
-			      5)){
-	sequencer = ags_distributed_manager_register_sequencer(AGS_DISTRIBUTED_MANAGER(core_audio_server),
-							       FALSE);
-
-	has_core_audio = TRUE;
-      }else if(!g_ascii_strncasecmp(str,
 			      "jack",
 			      5)){
-	sequencer = ags_distributed_manager_register_sequencer(AGS_DISTRIBUTED_MANAGER(jack_server),
+	sequencer = ags_sound_server_register_sequencer(AGS_SOUND_SERVER(jack_server),
 							       FALSE);
 
 	has_jack = TRUE;
@@ -1579,10 +1739,28 @@ ags_xorg_application_context_setup(AgsApplicationContext *application_context)
   }
 
   g_free(sequencer_group);
-    
+  
+  /* AgsWindow */
+#ifdef AGS_WITH_QUARTZ
+  g_object_new(GTKOSX_TYPE_APPLICATION,
+	       NULL);
+#endif
+  window = g_object_new(AGS_TYPE_WINDOW,
+			"soundcard", soundcard,
+			"application-context", xorg_application_context,
+			NULL);
+  g_object_set(xorg_application_context,
+	       "window", window,
+	       NULL);
+
+  gtk_window_set_default_size((GtkWindow *) window, 500, 500);
+  gtk_paned_set_position((GtkPaned *) window->paned, 300);
+
+  ags_connectable_connect(AGS_CONNECTABLE(window));
+
   /* stop animation */
-  ags_ui_provider_set_show_animation(AGS_UI_PROVIDER(xorg_application_context),
-				     FALSE);
+  g_atomic_int_set(&(xorg_application_context->show_animation),
+		   FALSE);  
 
   /* AgsServer */
   xorg_application_context->server = ags_server_new((GObject *) xorg_application_context);
@@ -1593,35 +1771,52 @@ ags_xorg_application_context_setup(AgsApplicationContext *application_context)
     
   while(list != NULL){
     AgsNotifySoundcard *notify_soundcard;
+
+    guint soundcard_capability;
+
+    soundcard_capability = ags_soundcard_get_capability(AGS_SOUNDCARD(list->data));
     
-    soundcard_thread = (AgsThread *) ags_soundcard_thread_new(list->data);
+    soundcard_thread = (AgsThread *) ags_soundcard_thread_new(list->data,
+							      soundcard_capability);
     ags_thread_add_child_extended(AGS_THREAD(audio_loop),
 				  (AgsThread *) soundcard_thread,
 				  TRUE, TRUE);
 
-    /* notify soundcard */
-    notify_soundcard = ags_notify_soundcard_new(soundcard_thread);
-    AGS_TASK(notify_soundcard)->task_thread = application_context->task_thread;
+    /* notify soundcard and export thread */
+    export_thread = NULL;
     
-    if(AGS_IS_DEVOUT(list->data)){
-      AGS_DEVOUT(list->data)->notify_soundcard = notify_soundcard;
-    }else if(AGS_IS_JACK_DEVOUT(list->data)){
-      AGS_JACK_DEVOUT(list->data)->notify_soundcard = notify_soundcard;
-    }else if(AGS_IS_PULSE_DEVOUT(list->data)){
-      AGS_PULSE_DEVOUT(list->data)->notify_soundcard = notify_soundcard;
-    }else if(AGS_IS_CORE_AUDIO_DEVOUT(list->data)){
-      AGS_CORE_AUDIO_DEVOUT(list->data)->notify_soundcard = notify_soundcard;
-    }
+    //    if(soundcard_capability == AGS_SOUNDCARD_CAPABILITY_PLAYBACK){
+      notify_soundcard = ags_notify_soundcard_new(soundcard_thread);
+      AGS_TASK(notify_soundcard)->task_thread = application_context->task_thread;
+    
+      if(AGS_IS_DEVOUT(list->data)){
+	AGS_DEVOUT(list->data)->notify_soundcard = notify_soundcard;
+      }else if(AGS_IS_JACK_DEVOUT(list->data)){
+	AGS_JACK_DEVOUT(list->data)->notify_soundcard = notify_soundcard;
+      }else if(AGS_IS_PULSE_DEVOUT(list->data)){
+	AGS_PULSE_DEVOUT(list->data)->notify_soundcard = notify_soundcard;
+      }else if(AGS_IS_CORE_AUDIO_DEVOUT(list->data)){
+	AGS_CORE_AUDIO_DEVOUT(list->data)->notify_soundcard = notify_soundcard;
+      }else if(AGS_IS_DEVIN(list->data)){
+	AGS_DEVIN(list->data)->notify_soundcard = notify_soundcard;
+      }else if(AGS_IS_JACK_DEVIN(list->data)){
+	AGS_JACK_DEVIN(list->data)->notify_soundcard = notify_soundcard;
+      }else if(AGS_IS_PULSE_DEVIN(list->data)){
+	AGS_PULSE_DEVIN(list->data)->notify_soundcard = notify_soundcard;
+      }else if(AGS_IS_CORE_AUDIO_DEVIN(list->data)){
+	AGS_CORE_AUDIO_DEVIN(list->data)->notify_soundcard = notify_soundcard;
+      }
 
-    ags_task_thread_append_cyclic_task(application_context->task_thread,
-				       notify_soundcard);
+      ags_task_thread_append_cyclic_task(application_context->task_thread,
+					 notify_soundcard);
 
-    /* export thread */
-    export_thread = (AgsThread *) ags_export_thread_new(list->data,
-							NULL);
-    ags_thread_add_child_extended(AGS_THREAD(audio_loop),
-				  (AgsThread *) export_thread,
-				  TRUE, TRUE);
+      /* export thread */
+      export_thread = (AgsThread *) ags_export_thread_new(list->data,
+							  NULL);
+      ags_thread_add_child_extended(AGS_THREAD(audio_loop),
+				    (AgsThread *) export_thread,
+				    TRUE, TRUE);
+      // }    
 
     /* default soundcard thread */
     if(xorg_application_context->soundcard_thread == NULL){
@@ -1630,8 +1825,10 @@ ags_xorg_application_context_setup(AgsApplicationContext *application_context)
     }
 
     /* default export thread */
-    if(export_thread != NULL){
+    if(export_thread != NULL &&
+       xorg_application_context->export_thread == NULL){
       xorg_application_context->export_thread = export_thread;
+      g_object_ref(export_thread);
     }
 
     /* iterate */
@@ -1698,44 +1895,10 @@ ags_xorg_application_context_setup(AgsApplicationContext *application_context)
   if(has_jack){
     ags_jack_server_connect_client(jack_server);
   }
-
-  /* emit message */
-  message_delivery = ags_message_delivery_get_instance();
-
-  message_queue = ags_message_delivery_find_namespace(message_delivery,
-						      "libags-audio");
-
-  if(message_queue != NULL){
-    AgsMessageEnvelope *message;
-
-    xmlDoc *doc;
-    xmlNode *root_node;
-
-    /* specify message body */
-    doc = xmlNewDoc("1.0");
-
-    root_node = xmlNewNode(NULL,
-			   "ags-command");
-    xmlDocSetRootElement(doc, root_node);    
-
-    xmlNewProp(root_node,
-	       "method",
-	       "AgsSoundProvider::config-read");
-
-    /* add message */
-    message = ags_message_envelope_alloc(application_context,
-					 NULL,
-					 doc);
-
-    /* add message */
-    ags_message_delivery_add_message(message_delivery,
-				     "libags-audio",
-				     message);
-  }
   
-  //  if(filename != NULL){
-  //    window->filename = filename;
-  //  }
+  if(filename != NULL){
+    window->filename = filename;
+  }
   //  pthread_mutex_unlock(ags_gui_thread_get_dispatch_mutex());
 }
 
@@ -1759,9 +1922,6 @@ ags_xorg_application_context_register_types(AgsApplicationContext *application_c
   ags_audio_loop_get_type();
   ags_soundcard_thread_get_type();
   ags_export_thread_get_type();
-  ags_record_thread_get_type();
-  ags_iterator_thread_get_type();
-  ags_recycling_thread_get_type();
 
   /*  */
   ags_audio_file_get_type();
@@ -1781,7 +1941,7 @@ ags_xorg_application_context_register_types(AgsApplicationContext *application_c
   //TODO:JK: extend me
   
   /* register recalls */
-  ags_recall_channel_run_dummy_get_type();
+  ags_generic_recall_channel_run_get_type();
 
   ags_play_audio_get_type();
   ags_play_channel_get_type();
@@ -1895,8 +2055,6 @@ ags_xorg_application_context_quit(AgsApplicationContext *application_context)
 
   AgsJackServer *jack_server;
 
-  AgsMutexManager *mutex_manager;
-
   AgsConfig *config;
 
   GList *core_audio_client;
@@ -1908,13 +2066,6 @@ ags_xorg_application_context_quit(AgsApplicationContext *application_context)
 
   gboolean autosave_thread_enabled;
 
-  pthread_mutex_t *application_mutex;
-
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);  
-
-  pthread_mutex_lock(application_mutex);
-  
   config = application_context->config;
   
   /* autosave thread */
@@ -1996,7 +2147,7 @@ ags_xorg_application_context_quit(AgsApplicationContext *application_context)
   }
 
   /* retrieve core audio server */
-  list = ags_sound_provider_get_distributed_manager(AGS_SOUND_PROVIDER(application_context));
+  list = ags_sound_provider_get_sound_server(AGS_SOUND_PROVIDER(application_context));
   
   while((list = ags_list_util_find_type(list,
 					AGS_TYPE_CORE_AUDIO_SERVER)) != NULL){
@@ -2017,7 +2168,7 @@ ags_xorg_application_context_quit(AgsApplicationContext *application_context)
   }
   
   /* retrieve pulseaudio server */
-  list = ags_sound_provider_get_distributed_manager(AGS_SOUND_PROVIDER(application_context));
+  list = ags_sound_provider_get_sound_server(AGS_SOUND_PROVIDER(application_context));
   
   while((list = ags_list_util_find_type(list,
 					AGS_TYPE_PULSE_SERVER)) != NULL){
@@ -2032,7 +2183,7 @@ ags_xorg_application_context_quit(AgsApplicationContext *application_context)
   }
   
   /* retrieve JACK server */
-  list = ags_sound_provider_get_distributed_manager(AGS_SOUND_PROVIDER(application_context));
+  list = ags_sound_provider_get_sound_server(AGS_SOUND_PROVIDER(application_context));
   
   if((list = ags_list_util_find_type(list,
 				     AGS_TYPE_JACK_SERVER)) != NULL){
@@ -2051,8 +2202,6 @@ ags_xorg_application_context_quit(AgsApplicationContext *application_context)
     
     list = list->next;
   }
-
-  pthread_mutex_unlock(application_mutex);
   
   gtk_main_quit();
 }
@@ -2152,26 +2301,14 @@ ags_xorg_application_context_read(AgsFile *file, xmlNode *node, GObject **applic
 void
 ags_xorg_application_context_launch(AgsFileLaunch *launch, AgsXorgApplicationContext *application_context)
 {
-  AgsMutexManager *mutex_manager;
-  
   AgsThread *audio_loop, *task_thread, *gui_thread;
 
   GList *list;
   GList *start_queue;
 
-  pthread_mutex_t *audio_loop_mutex;
-  pthread_mutex_t *application_mutex;
-
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
-    
-  pthread_mutex_lock(application_mutex);
-    
-  audio_loop = (AgsThread *) AGS_APPLICATION_CONTEXT(application_context)->main_loop;
-  audio_loop_mutex = ags_mutex_manager_lookup(mutex_manager,
-					      (GObject *) audio_loop);
-    
-  pthread_mutex_unlock(application_mutex);
+  g_object_get(application_context,
+	       "main-loop", &audio_loop,
+	       NULL);
 
   /* show all */
   gtk_widget_show_all((GtkWidget *) application_context->window);
@@ -2184,18 +2321,6 @@ ags_xorg_application_context_launch(AgsFileLaunch *launch, AgsXorgApplicationCon
   gui_thread = ags_thread_find_type(audio_loop,
 				    AGS_TYPE_GUI_THREAD);
 
-  pthread_mutex_lock(audio_loop_mutex);
-    
-  start_queue = NULL;
-  start_queue = g_list_prepend(start_queue,
-			       task_thread);
-  start_queue = g_list_prepend(start_queue,
-			       gui_thread);
-  g_atomic_pointer_set(&(audio_loop->start_queue),
-		       start_queue);
-
-  pthread_mutex_unlock(audio_loop_mutex);
-
   ags_thread_pool_start(application_context->thread_pool);
   ags_thread_start(audio_loop);
 }
