@@ -1,5 +1,5 @@
 /* GSequencer - Advanced GTK Sequencer
- * Copyright (C) 2005-2017 Joël Krähemann
+ * Copyright (C) 2005-2018 Joël Krähemann
  *
  * This file is part of GSequencer.
  *
@@ -44,8 +44,10 @@ void ags_lv2_bridge_get_property(GObject *gobject,
 				 GValue *value,
 				 GParamSpec *param_spec);
 void ags_lv2_bridge_finalize(GObject *gobject);
+
 void ags_lv2_bridge_connect(AgsConnectable *connectable);
 void ags_lv2_bridge_disconnect(AgsConnectable *connectable);
+
 gchar* ags_lv2_bridge_get_version(AgsPlugin *plugin);
 void ags_lv2_bridge_set_version(AgsPlugin *plugin, gchar *version);
 gchar* ags_lv2_bridge_get_build_id(AgsPlugin *plugin);
@@ -101,7 +103,7 @@ ags_lv2_bridge_get_type(void)
   static volatile gsize g_define_type_id__volatile = 0;
 
   if(g_once_init_enter (&g_define_type_id__volatile)){
-    GType ags_type_lv2_bridge;
+    GType ags_type_lv2_bridge = 0;
 
     static const GTypeInfo ags_lv2_bridge_info = {
       sizeof(AgsLv2BridgeClass),
@@ -139,7 +141,7 @@ ags_lv2_bridge_get_type(void)
 				AGS_TYPE_PLUGIN,
 				&ags_plugin_interface_info);
 
-    g_once_init_leave (&g_define_type_id__volatile, ags_type_lv2_bridge);
+    g_once_init_leave(&g_define_type_id__volatile, ags_type_lv2_bridge);
   }
 
   return g_define_type_id__volatile;
@@ -168,7 +170,7 @@ ags_lv2_bridge_class_init(AgsLv2BridgeClass *lv2_bridge)
    *
    * The plugin's filename.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   param_spec =  g_param_spec_string("filename",
 				    i18n_pspec("the object file"),
@@ -184,7 +186,7 @@ ags_lv2_bridge_class_init(AgsLv2BridgeClass *lv2_bridge)
    *
    * The effect's name.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   param_spec =  g_param_spec_string("effect",
 				    i18n_pspec("the effect"),
@@ -200,7 +202,7 @@ ags_lv2_bridge_class_init(AgsLv2BridgeClass *lv2_bridge)
    *
    * The uri's name.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   param_spec =  g_param_spec_string("uri",
 				    i18n_pspec("the uri"),
@@ -216,7 +218,7 @@ ags_lv2_bridge_class_init(AgsLv2BridgeClass *lv2_bridge)
    *
    * The uri's index.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   param_spec =  g_param_spec_ulong("index",
 				   i18n_pspec("index of uri"),
@@ -235,7 +237,7 @@ ags_lv2_bridge_class_init(AgsLv2BridgeClass *lv2_bridge)
    * If has-midi is set to %TRUE appropriate flag is set
    * to audio in order to become a sequencer.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   param_spec =  g_param_spec_boolean("has-midi",
 				     i18n_pspec("has-midi"),
@@ -252,7 +254,7 @@ ags_lv2_bridge_class_init(AgsLv2BridgeClass *lv2_bridge)
    * If has-gui is set to %TRUE 128 inputs are allocated and appropriate flag is set
    * to audio in order to become a sequencer.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   param_spec =  g_param_spec_boolean("has-gui",
 				     i18n_pspec("has-gui"),
@@ -268,7 +270,7 @@ ags_lv2_bridge_class_init(AgsLv2BridgeClass *lv2_bridge)
    *
    * The plugin's GUI filename.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   param_spec =  g_param_spec_string("gui-filename",
 				    i18n_pspec("the GUI object file"),
@@ -284,7 +286,7 @@ ags_lv2_bridge_class_init(AgsLv2BridgeClass *lv2_bridge)
    *
    * The GUI's uri name.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   param_spec =  g_param_spec_string("gui-uri",
 				    i18n_pspec("the gui-uri"),
@@ -353,7 +355,7 @@ ags_lv2_bridge_init(AgsLv2Bridge *lv2_bridge)
   }
 
   audio = AGS_MACHINE(lv2_bridge)->audio;
-  audio->flags |= (AGS_AUDIO_SYNC);
+  ags_audio_set_flags(audio, (AGS_AUDIO_SYNC));
 
   g_signal_connect_after(G_OBJECT(lv2_bridge), "resize-audio-channels",
 			 G_CALLBACK(ags_lv2_bridge_resize_audio_channels), NULL);
@@ -851,9 +853,9 @@ ags_lv2_bridge_launch_task(AgsFileLaunch *file_launch, AgsLv2Bridge *lv2_bridge)
     recall = AGS_MACHINE(lv2_bridge)->audio->input->recall;
     
     while((recall = ags_recall_template_find_type(recall, AGS_TYPE_RECALL_LV2)) != NULL){
-      if(!g_strcmp0(AGS_RECALL_LV2(recall->data)->filename,
+      if(!g_strcmp0(AGS_RECALL(recall->data)->filename,
 		  lv2_bridge->filename) &&
-	 !g_strcmp0(AGS_RECALL_LV2(recall->data)->effect,
+	 !g_strcmp0(AGS_RECALL(recall->data)->effect,
 		    lv2_bridge->effect)){
 	break;
       }
@@ -956,41 +958,21 @@ ags_lv2_bridge_resize_audio_channels(AgsMachine *machine,
   AgsRecycling *first_recycling;
   AgsAudioSignal *audio_signal;  
 
-  AgsMutexManager *mutex_manager;
-
-  GObject *soundcard;
+  GObject *output_soundcard;
   
   guint output_pads, input_pads;
-  
-  pthread_mutex_t *application_mutex;
-  pthread_mutex_t *audio_mutex;
-  pthread_mutex_t *channel_mutex;
-
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
 
   lv2_bridge = (AgsLv2Bridge *) machine;
 
   audio = machine->audio;
 
-  /* get audio mutex */
-  pthread_mutex_lock(application_mutex);
-
-  audio_mutex = ags_mutex_manager_lookup(mutex_manager,
-					 (GObject *) audio);
-  
-  pthread_mutex_unlock(application_mutex);  
-
   /* get some fields */
-  pthread_mutex_lock(audio_mutex);
-
-  output = audio->output;
-  input = audio->input;  
-
-  output_pads = audio->output_pads;
-  input_pads = audio->input_pads;
-
-  pthread_mutex_unlock(audio_mutex);
+  g_object_get(audio,
+	       "output", &output,
+	       "input", &input,
+	       "output-pads", &output_pads,
+	       "input-pads", &input_pads,
+	       NULL);
   
   /* check available */
   if(input_pads == 0 &&
@@ -1003,57 +985,37 @@ ags_lv2_bridge_resize_audio_channels(AgsMachine *machine,
     channel = input;
 
     while(channel != NULL){
-      /* get channel mutex */
-      pthread_mutex_lock(application_mutex);
-
-      channel_mutex = ags_mutex_manager_lookup(mutex_manager,
-					     (GObject *) channel);
-  
-      pthread_mutex_unlock(application_mutex);
-
       /* get some fields */
-      pthread_mutex_lock(channel_mutex);
-      
-      next_pad = channel->next_pad;
-
-      pthread_mutex_unlock(channel_mutex);
+      g_object_get(channel,
+		   "next-pad", &next_pad,
+		   NULL);
 
       channel = ags_channel_nth(channel,
 				audio_channels_old);
 
       while(channel != next_pad){
-	/* get channel mutex */
-	pthread_mutex_lock(application_mutex);
-
-	channel_mutex = ags_mutex_manager_lookup(mutex_manager,
-						 (GObject *) channel);
-  
-	pthread_mutex_unlock(application_mutex);
-
 	/* get some fields */
-	pthread_mutex_lock(channel_mutex);
-	
-      	soundcard = channel->soundcard;
-	first_recycling = channel->first_recycling;
+	g_object_get(channel,
+		     "output-soundcard", &output_soundcard,
+		     "first-recycling", &first_recycling,
+		     NULL);
 
-	pthread_mutex_unlock(channel_mutex);
-
-	/* audio signal */
-	audio_signal = ags_audio_signal_new(soundcard,
-					    (GObject *) first_recycling,
-					    NULL);
-	audio_signal->flags |= AGS_AUDIO_SIGNAL_TEMPLATE;
-	ags_audio_signal_stream_resize(audio_signal,
-				       1);
-	ags_recycling_add_audio_signal(first_recycling,
-				       audio_signal);
+	if(first_recycling != NULL){
+	  /* audio signal */
+	  audio_signal = ags_audio_signal_new(output_soundcard,
+					      (GObject *) first_recycling,
+					      NULL);
+	  audio_signal->flags |= AGS_AUDIO_SIGNAL_TEMPLATE;
+	  ags_audio_signal_stream_resize(audio_signal,
+					 1);
+	  ags_recycling_add_audio_signal(first_recycling,
+					 audio_signal);
+	}
 	
 	/* iterate */
-	pthread_mutex_lock(channel_mutex);
-      
-	channel = channel->next;
-
-	pthread_mutex_unlock(channel_mutex);
+	g_object_get(channel,
+		     "next", &channel,
+		     NULL);
       }
     }
 
@@ -1061,57 +1023,37 @@ ags_lv2_bridge_resize_audio_channels(AgsMachine *machine,
     channel = audio->output;
 
     while(channel != NULL){
-      /* get channel mutex */
-      pthread_mutex_lock(application_mutex);
-
-      channel_mutex = ags_mutex_manager_lookup(mutex_manager,
-					     (GObject *) channel);
-  
-      pthread_mutex_unlock(application_mutex);
-
       /* get some fields */
-      pthread_mutex_lock(channel_mutex);
-      
-      next_pad = channel->next_pad;
-
-      pthread_mutex_unlock(channel_mutex);
+      g_object_get(channel,
+		   "next-pad", &next_pad,
+		   NULL);
 
       channel = ags_channel_pad_nth(channel,
 				    audio_channels_old);
 
       while(channel != next_pad){
-	/* get channel mutex */
-	pthread_mutex_lock(application_mutex);
-
-	channel_mutex = ags_mutex_manager_lookup(mutex_manager,
-						 (GObject *) channel);
-  
-	pthread_mutex_unlock(application_mutex);
-
 	/* get some fields */
-	pthread_mutex_lock(channel_mutex);
-	
-      	soundcard = channel->soundcard;
-	first_recycling = channel->first_recycling;
+	g_object_get(channel,
+		     "output-soundcard", &output_soundcard,
+		     "first-recycling", &first_recycling,
+		     NULL);
 
-	pthread_mutex_unlock(channel_mutex);
-
-	/* audio signal */
-	audio_signal = ags_audio_signal_new(soundcard,
-					    (GObject *) first_recycling,
-					    NULL);
-	audio_signal->flags |= AGS_AUDIO_SIGNAL_TEMPLATE;
-	ags_audio_signal_stream_resize(audio_signal,
-				       3);
-	ags_recycling_add_audio_signal(first_recycling,
-				       audio_signal);
+	if(first_recycling != NULL){
+	  /* audio signal */
+	  audio_signal = ags_audio_signal_new(output_soundcard,
+					      (GObject *) first_recycling,
+					      NULL);
+	  audio_signal->flags |= AGS_AUDIO_SIGNAL_TEMPLATE;
+	  ags_audio_signal_stream_resize(audio_signal,
+					 3);
+	  ags_recycling_add_audio_signal(first_recycling,
+					 audio_signal);
+	}
 	
 	/* iterate */
-	pthread_mutex_lock(channel_mutex);
-      
-	channel = channel->next;
-
-	pthread_mutex_unlock(channel_mutex);
+	g_object_get(channel,
+		     "next", &channel,
+		     NULL);
       }
     }
 
@@ -1141,41 +1083,21 @@ ags_lv2_bridge_resize_pads(AgsMachine *machine, GType channel_type,
   AgsRecycling *first_recycling;
   AgsAudioSignal *audio_signal;
   
-  AgsMutexManager *mutex_manager;
-
-  GObject *soundcard;
+  GObject *output_soundcard;
   
   guint audio_channels;
   gboolean grow;
 
-  pthread_mutex_t *application_mutex;
-  pthread_mutex_t *audio_mutex;
-  pthread_mutex_t *channel_mutex;
-
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
-
   lv2_bridge = (AgsLv2Bridge *) machine;
 
   audio = machine->audio;  
 
-  /* get audio mutex */
-  pthread_mutex_lock(application_mutex);
-
-  audio_mutex = ags_mutex_manager_lookup(mutex_manager,
-					 (GObject *) audio);
-  
-  pthread_mutex_unlock(application_mutex);  
-
   /* get some fields */
-  pthread_mutex_lock(audio_mutex);
-
-  output = audio->output;
-  input = audio->input;
-  
-  audio_channels = audio->audio_channels;
-
-  pthread_mutex_unlock(audio_mutex);
+  g_object_get(audio,
+	       "output", &output,
+	       "input", &input,
+	       "audio-channels", &audio_channels,
+	       NULL);
 
   /* check available */
   if(pads == pads_old ||
@@ -1196,39 +1118,29 @@ ags_lv2_bridge_resize_pads(AgsMachine *machine, GType channel_type,
 				    pads_old);
 
       while(channel != NULL){
-	/* get channel mutex */
-	pthread_mutex_lock(application_mutex);
-
-	channel_mutex = ags_mutex_manager_lookup(mutex_manager,
-						 (GObject *) channel);
-  
-	pthread_mutex_unlock(application_mutex);
-
 	/* get some fields */
-	pthread_mutex_lock(channel_mutex);
-	
-	soundcard = channel->soundcard;
+	g_object_get(channel,
+		     "output-soundcard", &output_soundcard,
+		     "first-recycling", &first_recycling,
+		     NULL);
 
-	first_recycling = channel->first_recycling;
 
-	pthread_mutex_unlock(channel_mutex);
-
-	/* audio signal */
-	audio_signal = ags_audio_signal_new(soundcard,
-					    (GObject *) first_recycling,
-					    NULL);
-	audio_signal->flags |= AGS_AUDIO_SIGNAL_TEMPLATE;
-	ags_audio_signal_stream_resize(audio_signal,
-				       1);
-	ags_recycling_add_audio_signal(first_recycling,
-				       audio_signal);
+	if(first_recycling != NULL){
+	  /* audio signal */
+	  audio_signal = ags_audio_signal_new(output_soundcard,
+					      (GObject *) first_recycling,
+					      NULL);
+	  audio_signal->flags |= AGS_AUDIO_SIGNAL_TEMPLATE;
+	  ags_audio_signal_stream_resize(audio_signal,
+					 1);
+	  ags_recycling_add_audio_signal(first_recycling,
+					 audio_signal);
+	}
 	
 	/* iterate */
-	pthread_mutex_lock(channel_mutex);
-
-	channel = channel->next;
-
-	pthread_mutex_unlock(channel_mutex);
+	g_object_get(channel,
+		     "next", &channel,
+		     NULL);
       }
 
       /* recall */
@@ -1247,39 +1159,28 @@ ags_lv2_bridge_resize_pads(AgsMachine *machine, GType channel_type,
 				    pads_old);
 
       while(channel != NULL){
-	/* get channel mutex */
-	pthread_mutex_lock(application_mutex);
-
-	channel_mutex = ags_mutex_manager_lookup(mutex_manager,
-						 (GObject *) channel);
-  
-	pthread_mutex_unlock(application_mutex);
-
 	/* get some fields */
-	pthread_mutex_lock(channel_mutex);
-	
-	soundcard = channel->soundcard;
+	g_object_get(channel,
+		     "output-soundcard", &output_soundcard,
+		     "first-recycling", &first_recycling,
+		     NULL);
 
-	first_recycling = channel->first_recycling;
-
-	pthread_mutex_unlock(channel_mutex);
-
-	/* audio signal */
-	audio_signal = ags_audio_signal_new(soundcard,
-					    (GObject *) first_recycling,
-					    NULL);
-	audio_signal->flags |= AGS_AUDIO_SIGNAL_TEMPLATE;
-	ags_audio_signal_stream_resize(audio_signal,
-				       3);
-	ags_recycling_add_audio_signal(first_recycling,
-				       audio_signal);
+	if(first_recycling != NULL){
+	  /* audio signal */
+	  audio_signal = ags_audio_signal_new(output_soundcard,
+					      (GObject *) first_recycling,
+					      NULL);
+	  audio_signal->flags |= AGS_AUDIO_SIGNAL_TEMPLATE;
+	  ags_audio_signal_stream_resize(audio_signal,
+					 3);
+	  ags_recycling_add_audio_signal(first_recycling,
+					 audio_signal);
+	}
 	
 	/* iterate */
-	pthread_mutex_lock(channel_mutex);
-
-	channel = channel->next;
-
-	pthread_mutex_unlock(channel_mutex);
+	g_object_get(channel,
+		     "next", &channel,
+		     NULL);
       }
 
       /* recall */
@@ -1313,21 +1214,15 @@ ags_lv2_bridge_map_recall(AgsMachine *machine)
   AgsRouteLv2Audio *recall_route_lv2_audio;
   AgsRouteLv2AudioRun *recall_route_lv2_audio_run;
 
-  AgsMutexManager *mutex_manager;
-
   GList *list;
-
-  pthread_mutex_t *application_mutex;
-  pthread_mutex_t *audio_mutex;
+  GList *start_play, *play;
+  GList *start_recall, *recall;
   
   if((AGS_MACHINE_MAPPED_RECALL & (machine->flags)) != 0 ||
      (AGS_MACHINE_PREMAPPED_RECALL & (machine->flags)) != 0){
     return;
   }
 
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
-
   window = (AgsWindow *) gtk_widget_get_ancestor((GtkWidget *) machine,
 						 AGS_TYPE_WINDOW);
 
@@ -1335,14 +1230,10 @@ ags_lv2_bridge_map_recall(AgsMachine *machine)
 
   audio = machine->audio;
 
-  /* get audio mutex */
-  pthread_mutex_lock(application_mutex);
-
-  audio_mutex = ags_mutex_manager_lookup(mutex_manager,
-					 (GObject *) audio);
+  g_object_get(audio,
+	       "play", &start_play,
+	       NULL);
   
-  pthread_mutex_unlock(application_mutex);  
-
   /* ags-delay */
   if((AGS_MACHINE_IS_SYNTHESIZER & (machine->flags)) != 0){
     ags_recall_factory_create(audio,
@@ -1356,19 +1247,15 @@ ags_lv2_bridge_map_recall(AgsMachine *machine)
 			       AGS_RECALL_FACTORY_RECALL),
 			      0);
 
-    pthread_mutex_lock(audio_mutex);
-    
-    list = ags_recall_find_type(audio->play,
+    play = ags_recall_find_type(start_play,
 				AGS_TYPE_DELAY_AUDIO_RUN);
 
-    if(list != NULL){
-      play_delay_audio_run = AGS_DELAY_AUDIO_RUN(list->data);
+    if(play != NULL){
+      play_delay_audio_run = AGS_DELAY_AUDIO_RUN(play->data);
       //    AGS_RECALL(play_delay_audio_run)->flags |= AGS_RECALL_PERSISTENT;
     }else{
       play_delay_audio_run = NULL;
     }
-
-    pthread_mutex_unlock(audio_mutex);
     
     /* ags-count-beats */
     ags_recall_factory_create(audio,
@@ -1382,13 +1269,11 @@ ags_lv2_bridge_map_recall(AgsMachine *machine)
 			       AGS_RECALL_FACTORY_RECALL),
 			      0);
   
-    pthread_mutex_lock(audio_mutex);
-    
-    list = ags_recall_find_type(audio->play,
+    play = ags_recall_find_type(start_play,
 				AGS_TYPE_COUNT_BEATS_AUDIO_RUN);
 
-    if(list != NULL){
-      play_count_beats_audio_run = AGS_COUNT_BEATS_AUDIO_RUN(list->data);
+    if(play != NULL){
+      play_count_beats_audio_run = AGS_COUNT_BEATS_AUDIO_RUN(play->data);
 
       /* set dependency */  
       g_object_set(G_OBJECT(play_count_beats_audio_run),
@@ -1401,8 +1286,6 @@ ags_lv2_bridge_map_recall(AgsMachine *machine)
       play_count_beats_audio_run = NULL;
     }
 
-    pthread_mutex_unlock(audio_mutex);
-
     /* ags-record-midi */
     ags_recall_factory_create(audio,
 			      NULL, NULL,
@@ -1414,13 +1297,11 @@ ags_lv2_bridge_map_recall(AgsMachine *machine)
 			       AGS_RECALL_FACTORY_RECALL),
 			      0);
 
-    pthread_mutex_lock(audio_mutex);
+    recall = ags_recall_find_type(start_recall,
+				  AGS_TYPE_RECORD_MIDI_AUDIO_RUN);
 
-    list = ags_recall_find_type(audio->recall,
-				AGS_TYPE_RECORD_MIDI_AUDIO_RUN);
-
-    if(list != NULL){
-      recall_record_midi_audio_run = AGS_RECORD_MIDI_AUDIO_RUN(list->data);
+    if(recall != NULL){
+      recall_record_midi_audio_run = AGS_RECORD_MIDI_AUDIO_RUN(recall->data);
     
       /* set dependency */
       g_object_set(G_OBJECT(recall_record_midi_audio_run),
@@ -1433,8 +1314,6 @@ ags_lv2_bridge_map_recall(AgsMachine *machine)
 		   NULL);
     }  
 
-    pthread_mutex_unlock(audio_mutex);
-
     /* ags-route-lv2 */
     ags_recall_factory_create(audio,
 			      NULL, NULL,
@@ -1445,14 +1324,12 @@ ags_lv2_bridge_map_recall(AgsMachine *machine)
 			       AGS_RECALL_FACTORY_ADD |
 			       AGS_RECALL_FACTORY_RECALL),
 			      0);
-
-    pthread_mutex_lock(audio_mutex);
     
-    list = ags_recall_find_type(audio->recall,
-				AGS_TYPE_ROUTE_LV2_AUDIO_RUN);
+    recall = ags_recall_find_type(start_recall,
+				  AGS_TYPE_ROUTE_LV2_AUDIO_RUN);
 
-    if(list != NULL){
-      recall_route_lv2_audio_run = AGS_ROUTE_LV2_AUDIO_RUN(list->data);
+    if(recall != NULL){
+      recall_route_lv2_audio_run = AGS_ROUTE_LV2_AUDIO_RUN(recall->data);
 
       /* set dependency */
       g_object_set(G_OBJECT(recall_route_lv2_audio_run),
@@ -1467,8 +1344,6 @@ ags_lv2_bridge_map_recall(AgsMachine *machine)
       recall_route_lv2_audio_run = NULL;
     }
 
-    pthread_mutex_unlock(audio_mutex);
-
     /* ags-play-notation */
     ags_recall_factory_create(audio,
 			      NULL, NULL,
@@ -1479,14 +1354,12 @@ ags_lv2_bridge_map_recall(AgsMachine *machine)
 			       AGS_RECALL_FACTORY_ADD |
 			       AGS_RECALL_FACTORY_RECALL),
 			      0);
-
-    pthread_mutex_lock(audio_mutex);
     
-    list = ags_recall_find_type(audio->recall,
-				AGS_TYPE_PLAY_NOTATION_AUDIO_RUN);
+    recall = ags_recall_find_type(start_recall,
+				  AGS_TYPE_PLAY_NOTATION_AUDIO_RUN);
 
-    if(list != NULL){
-      recall_notation_audio_run = AGS_PLAY_NOTATION_AUDIO_RUN(list->data);
+    if(recall != NULL){
+      recall_notation_audio_run = AGS_PLAY_NOTATION_AUDIO_RUN(recall->data);
 
       /* set dependency */
       g_object_set(G_OBJECT(recall_notation_audio_run),
@@ -1498,9 +1371,10 @@ ags_lv2_bridge_map_recall(AgsMachine *machine)
 		   "count-beats-audio-run", play_count_beats_audio_run,
 		   NULL);
     }
-
-    pthread_mutex_unlock(audio_mutex);
   }
+
+  g_list_free(start_play);
+  g_list_free(start_recall);
   
   /* depending on destination */
   ags_lv2_bridge_input_map_recall(lv2_bridge,
@@ -1531,51 +1405,21 @@ ags_lv2_bridge_input_map_recall(AgsLv2Bridge *lv2_bridge,
   AgsChannel *input;
   AgsChannel *source, *current;
   
-  AgsMutexManager *mutex_manager;
-  
-  AgsConfig *config;
-  
-  GList *list;
-
-  gchar *str;
-
   guint input_pads;
   guint audio_channels;
-  gboolean rt_safe;
-  gboolean performance_mode;
-
-  pthread_mutex_t *application_mutex;
-  pthread_mutex_t *audio_mutex;
-  pthread_mutex_t *current_mutex;
 
   if(lv2_bridge->mapped_input_pad > input_pad_start){
     return;
   }
 
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
-
-  config = ags_config_get_instance();
-
   audio = AGS_MACHINE(lv2_bridge)->audio;
 
-  /* get audio mutex */
-  pthread_mutex_lock(application_mutex);
-
-  audio_mutex = ags_mutex_manager_lookup(mutex_manager,
-					 (GObject *) audio);
-  
-  pthread_mutex_unlock(application_mutex);
-
   /* get some fields */
-  pthread_mutex_lock(audio_mutex);
-
-  input = audio->input;
-
-  input_pads = audio->input_pads;
-  audio_channels = audio->audio_channels;
-  
-  pthread_mutex_unlock(audio_mutex);
+  g_object_get(audio,
+	       "input", &input,
+	       "input-pads", &input_pads,
+	       "audio-channels", &audio_channels,
+	       NULL);
 
   /* source */
   source = ags_channel_nth(input,
@@ -1597,81 +1441,76 @@ ags_lv2_bridge_input_map_recall(AgsLv2Bridge *lv2_bridge,
     current = source;
 
     while(current != NULL){
-      /* get current mutex */
-      pthread_mutex_lock(application_mutex);
-
-      current_mutex = ags_mutex_manager_lookup(mutex_manager,
-					       (GObject *) current);
-  
-      pthread_mutex_unlock(application_mutex);
-
-      pthread_mutex_lock(current_mutex);
+      AgsPort *port;
+      
+      GList *start_list, *list;
       
       /* play - use note length */
-      list = ags_recall_template_find_type(current->play,
+      g_object_get(current,
+		   "play", &start_list,
+		   NULL);
+
+      list = ags_recall_template_find_type(start_list,
 					   AGS_TYPE_ENVELOPE_CHANNEL);
 
       if(list != NULL){
 	GValue use_note_length_value = {0,};
 
-	g_value_init(&use_note_length_value, G_TYPE_BOOLEAN);
+	g_value_init(&use_note_length_value,
+		     G_TYPE_BOOLEAN);
+	
 	g_value_set_boolean(&use_note_length_value,
 			    TRUE);
 
-	ags_port_safe_write(AGS_ENVELOPE_CHANNEL(list->data)->use_note_length,
+	g_object_get(list->data,
+		     "use-note-length", &port,
+		     NULL);
+	
+	ags_port_safe_write(port,
 			    &use_note_length_value);
+
+	g_value_unset(&use_note_length_value);
       }
+
+      g_list_free(start_list);
       
       /* recall - use note length */
+      g_object_get(current,
+		   "recall", &start_list,
+		   NULL);
+
       list = ags_recall_template_find_type(current->recall,
 					   AGS_TYPE_ENVELOPE_CHANNEL);
 
       if(list != NULL){
 	GValue use_note_length_value = {0,};
 
-	g_value_init(&use_note_length_value, G_TYPE_BOOLEAN);
+	g_value_init(&use_note_length_value,
+		     G_TYPE_BOOLEAN);
+	
 	g_value_set_boolean(&use_note_length_value,
 			    TRUE);
 
-	ags_port_safe_write(AGS_ENVELOPE_CHANNEL(list->data)->use_note_length,
+	g_object_get(list->data,
+		     "use-note-length", &port,
+		     NULL);
+
+	ags_port_safe_write(port,
 			    &use_note_length_value);
+
+	g_value_unset(&use_note_length_value);
       }
 
-      current = current->next_pad;
+      g_list_free(start_list);
 
-      pthread_mutex_unlock(current_mutex);
-    }
-
-    /* map dependending on output */
-    rt_safe = TRUE;
-    performance_mode = TRUE;
-
-    str = ags_config_get_value(config,
-			       AGS_CONFIG_GENERIC,
-			       "rt-safe");
-
-    if(str != NULL &&
-       !g_ascii_strncasecmp(str,
-			    "FALSE",
-			    6)){
-      rt_safe = FALSE;
-    }
-
-    str = ags_config_get_value(config,
-			       AGS_CONFIG_GENERIC,
-			       "engine-mode");
-
-    if(str != NULL &&
-       !g_ascii_strncasecmp(str,
-			    "performance",
-			    12)){
-      performance_mode = TRUE;
-    }else{
-      performance_mode = FALSE;
+      /* iterate */
+      g_object_get(current,
+		   "next-pad", &current,
+		   NULL);
     }
     
-    if(rt_safe ||
-       performance_mode){
+    if(ags_recall_global_get_rt_safe() ||
+       ags_recall_global_get_performance_mode()){
       /* ags-copy */
       ags_recall_factory_create(audio,
 				NULL, NULL,
@@ -1684,9 +1523,6 @@ ags_lv2_bridge_input_map_recall(AgsLv2Bridge *lv2_bridge,
 				0);
 
       current = current->next_pad;
-
-      /* set performance mode */
-      performance_mode = TRUE;
     }else{    
       /* ags-buffer */
       ags_recall_factory_create(audio,
@@ -1711,7 +1547,8 @@ ags_lv2_bridge_input_map_recall(AgsLv2Bridge *lv2_bridge,
 			       AGS_RECALL_FACTORY_ADD),
 			      0);
     /* ags-stream */
-    if(!rt_safe){
+    if(!(ags_recall_global_get_rt_safe() ||
+	 ags_recall_global_get_performance_mode())){
       ags_recall_factory_create(audio,
 				NULL, NULL,
 				"ags-stream",
@@ -1750,83 +1587,26 @@ ags_lv2_bridge_output_map_recall(AgsLv2Bridge *lv2_bridge,
   AgsDelayAudio *recall_delay_audio;
   AgsCountBeatsAudioRun *recall_count_beats_audio_run;
 
-  AgsMutexManager *mutex_manager;
-
-  AgsConfig *config;
-
-  GList *list;
-
-  gchar *str;
-
   guint output_pads, input_pads;
   guint audio_channels;
-  gboolean rt_safe;
-  gboolean performance_mode;
-
-  pthread_mutex_t *application_mutex;
-  pthread_mutex_t *audio_mutex;
 
   if(lv2_bridge->mapped_output_pad > output_pad_start){
     return;
   }
 
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
-
-  config = ags_config_get_instance();
-
   audio = AGS_MACHINE(lv2_bridge)->audio;
 
-  /* get audio mutex */
-  pthread_mutex_lock(application_mutex);
-
-  audio_mutex = ags_mutex_manager_lookup(mutex_manager,
-					 (GObject *) audio);
-  
-  pthread_mutex_unlock(application_mutex);  
-
   /* get some fields */
-  pthread_mutex_lock(audio_mutex);
+  g_object_get(audio,
+	       "input-pads", &input_pads,
+	       "output-pads", &output_pads,
+	       "audio-channels", &audio_channels,
+	       NULL);
 
-  output_pads = audio->output_pads;
-  input_pads = audio->input_pads;
-  
-  audio_channels = audio->audio_channels;
-
-  pthread_mutex_unlock(audio_mutex);
-
-  if((AGS_MACHINE_IS_SYNTHESIZER & (AGS_MACHINE(lv2_bridge)->flags)) != 0){
-    /* map dependending on output */
-    rt_safe = TRUE;
-    performance_mode = TRUE;
-
-    str = ags_config_get_value(config,
-			       AGS_CONFIG_GENERIC,
-			       "rt-safe");
-
-    if(str != NULL &&
-       !g_ascii_strncasecmp(str,
-			    "FALSE",
-			    6)){
-      rt_safe = FALSE;
-    }
-
-    str = ags_config_get_value(config,
-			       AGS_CONFIG_GENERIC,
-			       "engine-mode");
-
-    if(str != NULL &&
-       !g_ascii_strncasecmp(str,
-			    "performance",
-			    12)){
-      performance_mode = TRUE;
-    }else{
-      performance_mode = FALSE;
-    }
-  
+  if((AGS_MACHINE_IS_SYNTHESIZER & (AGS_MACHINE(lv2_bridge)->flags)) != 0){  
     /* remap for input */
-    if(rt_safe ||
-       performance_mode){
+    if(ags_recall_global_get_rt_safe() ||
+       ags_recall_global_get_performance_mode()){
       /* ags-copy */
       ags_recall_factory_create(audio,
 				NULL, NULL,
@@ -1837,9 +1617,6 @@ ags_lv2_bridge_output_map_recall(AgsLv2Bridge *lv2_bridge,
 				 AGS_RECALL_FACTORY_RECALL |
 				 AGS_RECALL_FACTORY_REMAP),
 				0);
-
-      /* set performance mode */
-      performance_mode = TRUE;
     }else{    
       /* ags-buffer */
       ags_recall_factory_create(audio,
@@ -1854,7 +1631,8 @@ ags_lv2_bridge_output_map_recall(AgsLv2Bridge *lv2_bridge,
     }
     
     /*  */
-    if(!performance_mode){
+    if(!(ags_recall_global_get_rt_safe() ||
+	 ags_recall_global_get_performance_mode())){
       /* ags-stream */
       ags_recall_factory_create(audio,
 				NULL, NULL,
@@ -1898,34 +1676,46 @@ ags_lv2_bridge_load_program(AgsLv2Bridge *lv2_bridge)
     uint32_t i;
 
     if(lv2_bridge->lv2_handle == NULL){
+      guint samplerate;
+      guint buffer_size;
+
+      g_object_get(AGS_MACHINE(lv2_bridge)->audio,
+		   "samplerate", &samplerate,
+		   "buffer-size", &buffer_size,
+		   NULL);
+      
       lv2_bridge->lv2_handle = ags_base_plugin_instantiate(lv2_plugin,
-							   AGS_MACHINE(lv2_bridge)->audio->samplerate);
+							   samplerate, buffer_size);
     }
     
     if(lv2_bridge->port_value == NULL){
-      GList *port_descriptor;
+      GList *start_plugin_port, *plugin_port;
       
       guint port_count;
 
-      port_count = g_list_length(AGS_BASE_PLUGIN(lv2_plugin)->port);
+      g_object_get(lv2_plugin,
+		   "plugin-port", &start_plugin_port,
+		   NULL);
+
+      plugin_port = start_plugin_port;
+      port_count = g_list_length(start_plugin_port);
 
       if(port_count > 0){
 	lv2_bridge->port_value = (float *) malloc(port_count * sizeof(float));
       }
-
-      port_descriptor = AGS_BASE_PLUGIN(lv2_plugin)->port;
       
-      for(i = 0; port_descriptor != NULL;){
-	if((AGS_PORT_DESCRIPTOR_CONTROL & (AGS_PORT_DESCRIPTOR(port_descriptor->data)->flags)) != 0){
-	  if((AGS_PORT_DESCRIPTOR_INPUT & (AGS_PORT_DESCRIPTOR(port_descriptor->data)->flags)) != 0){
+      for(i = 0; plugin_port != NULL;){
+	if(ags_plugin_port_test_flags(plugin_port->data, AGS_PLUGIN_PORT_CONTROL)){
+	  if(ags_plugin_port_test_flags(plugin_port->data, AGS_PLUGIN_PORT_INPUT)){
 	    plugin_descriptor->connect_port(lv2_bridge->lv2_handle[0],
-					    AGS_PORT_DESCRIPTOR(port_descriptor->data)->port_index,
+					    AGS_PLUGIN_PORT(plugin_port->data)->port_index,
 					    &(lv2_bridge->port_value[i]));
 	    i++;
 	  }
 	}
 
-	port_descriptor = port_descriptor->next;
+	/* iterate */
+	plugin_port = plugin_port->next;
       }
     }
 
@@ -2091,6 +1881,9 @@ ags_lv2_bridge_load(AgsLv2Bridge *lv2_bridge)
 {
   AgsLv2Plugin *lv2_plugin;
     
+  guint samplerate;
+  guint buffer_size;
+
   lv2_plugin = ags_lv2_manager_find_lv2_plugin(ags_lv2_manager_get_instance(),
 					       lv2_bridge->filename,
 					       lv2_bridge->effect);
@@ -2104,9 +1897,13 @@ ags_lv2_bridge_load(AgsLv2Bridge *lv2_bridge)
 	       "uri", lv2_plugin->uri,
 	       NULL);
 
+  /* samplerate and buffer size */
+  samplerate = ags_soundcard_helper_config_get_samplerate(ags_config_get_instance());
+  buffer_size = ags_soundcard_helper_config_get_buffer_size(ags_config_get_instance());
+
   /* program */
   lv2_bridge->lv2_handle = ags_base_plugin_instantiate(lv2_plugin,
-						       AGS_MACHINE(lv2_bridge)->audio->samplerate);
+						       samplerate, buffer_size);
 
   if((AGS_LV2_PLUGIN_HAS_PROGRAM_INTERFACE & (lv2_plugin->flags)) != 0){
     ags_lv2_bridge_load_program(lv2_bridge);
@@ -2129,7 +1926,7 @@ ags_lv2_bridge_load(AgsLv2Bridge *lv2_bridge)
  *
  * Returns: %TRUE if proceed with redraw, otherwise %FALSE
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 gboolean
 ags_lv2_bridge_lv2ui_idle_timeout(GtkWidget *widget)
@@ -2166,11 +1963,11 @@ ags_lv2_bridge_lv2ui_idle_timeout(GtkWidget *widget)
  * @filename: the plugin.so
  * @effect: the effect
  *
- * Creates an #AgsLv2Bridge
+ * Create a new instance of #AgsLv2Bridge
  *
- * Returns: a new #AgsLv2Bridge
+ * Returns: the new #AgsLv2Bridge
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 AgsLv2Bridge*
 ags_lv2_bridge_new(GObject *soundcard,
@@ -2183,14 +1980,10 @@ ags_lv2_bridge_new(GObject *soundcard,
   lv2_bridge = (AgsLv2Bridge *) g_object_new(AGS_TYPE_LV2_BRIDGE,
 					     NULL);
 
-  if(soundcard != NULL){
-    g_value_init(&value, G_TYPE_OBJECT);
-    g_value_set_object(&value, soundcard);
-    g_object_set_property(G_OBJECT(AGS_MACHINE(lv2_bridge)->audio),
-			  "soundcard", &value);
-    g_value_unset(&value);
-  }
-
+  g_object_set(AGS_MACHINE(lv2_bridge)->audio,
+	       "output-soundcard", soundcard,
+	       NULL);
+  
   g_object_set(lv2_bridge,
 	       "filename", filename,
 	       "effect", effect,
