@@ -1,5 +1,5 @@
 /* GSequencer - Advanced GTK Sequencer
- * Copyright (C) 2005-2015 Joël Krähemann
+ * Copyright (C) 2005-2018 Joël Krähemann
  *
  * This file is part of GSequencer.
  *
@@ -19,12 +19,12 @@
 
 #include <ags/plugin/ags_base_plugin.h>
 
-#include <ags/object/ags_marshal.h>
+#include <ags/libags.h>
 
 #include <stdlib.h>
 #include <string.h>
 
-#include <pthread.h>
+#include <ags/plugin/ags_plugin_port.h>
 
 #include <ags/i18n.h>
 
@@ -63,29 +63,32 @@ enum{
 
 enum{
   PROP_0,
-  PROP_ID,
+  PROP_UUID,
   PROP_FILENAME,
   PROP_EFFECT,
+  PROP_PLUGIN_PORT,
   PROP_EFFECT_INDEX,
+  PROP_PLUGIN_SO,
   PROP_UI_FILENAME,
   PROP_UI_EFFECT,
   PROP_UI_EFFECT_INDEX,  
+  PROP_UI_PLUGIN_SO,
   PROP_UI_PLUGIN,
 };
 
-pthread_mutex_t ags_base_plugin_class_mutex = PTHREAD_MUTEX_INITIALIZER;
-
 static gpointer ags_base_plugin_parent_class = NULL;
 static guint base_plugin_signals[LAST_SIGNAL];
 
+static pthread_mutex_t ags_base_plugin_class_mutex = PTHREAD_MUTEX_INITIALIZER;
+
 GType
-ags_base_plugin_get_type()
+ags_base_plugin_get_type (void)
 {
   static volatile gsize g_define_type_id__volatile = 0;
 
   if(g_once_init_enter (&g_define_type_id__volatile)){
-    GType ags_type_base_plugin;
-    
+    GType ags_type_base_plugin = 0;
+
     static const GTypeInfo ags_base_plugin_info = {
       sizeof(AgsBasePluginClass),
       NULL, /* base_init */
@@ -103,7 +106,7 @@ ags_base_plugin_get_type()
 						  &ags_base_plugin_info,
 						  0);
 
-    g_once_init_leave (&g_define_type_id__volatile, ags_type_base_plugin);
+    g_once_init_leave(&g_define_type_id__volatile, ags_type_base_plugin);
   }
 
   return g_define_type_id__volatile;
@@ -113,6 +116,7 @@ void
 ags_base_plugin_class_init(AgsBasePluginClass *base_plugin)
 {
   GObjectClass *gobject;
+
   GParamSpec *param_spec;
   
   ags_base_plugin_parent_class = g_type_class_peek_parent(base_plugin);
@@ -128,19 +132,18 @@ ags_base_plugin_class_init(AgsBasePluginClass *base_plugin)
 
   /* properties */
   /**
-   * AgsBasePlugin:id:
+   * AgsBasePlugin:uuid:
    *
-   * The assigned id.
+   * The assigned AgsUUID
    * 
-   * Since: 1.4.21
+   * Since: 2.0.0
    */
-  param_spec = g_param_spec_string("id",
-				   i18n_pspec("id of the plugin"),
-				   i18n_pspec("The id this plugin is located in"),
-				   NULL,
-				   G_PARAM_READABLE | G_PARAM_WRITABLE);
+  param_spec = g_param_spec_pointer("uuid",
+				    i18n_pspec("UUID"),
+				    i18n_pspec("The UUID"),
+				    G_PARAM_READABLE | G_PARAM_WRITABLE);
   g_object_class_install_property(gobject,
-				  PROP_ID,
+				  PROP_UUID,
 				  param_spec);
 
   /**
@@ -148,7 +151,7 @@ ags_base_plugin_class_init(AgsBasePluginClass *base_plugin)
    *
    * The assigned filename.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_string("filename",
 				   i18n_pspec("filename of the plugin"),
@@ -164,7 +167,7 @@ ags_base_plugin_class_init(AgsBasePluginClass *base_plugin)
    *
    * The assigned effect.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_string("effect",
 				   i18n_pspec("effect of the plugin"),
@@ -175,30 +178,60 @@ ags_base_plugin_class_init(AgsBasePluginClass *base_plugin)
 				  PROP_EFFECT,
 				  param_spec);
 
+  /**
+   * AgsBasePlugin:plugin-port:
+   *
+   * The assigned #GList-struct containing #AgsPluginPort
+   * 
+   * Since: 2.0.0
+   */
+  param_spec = g_param_spec_pointer("plugin-port",
+				    i18n_pspec("plugin port of base plugin"),
+				    i18n_pspec("The plugin port of base plugin"),
+				    G_PARAM_READABLE | G_PARAM_WRITABLE);
+  g_object_class_install_property(gobject,
+				  PROP_PLUGIN_PORT,
+				  param_spec);
+  
   /**
    * AgsBasePlugin:effect-index:
    *
    * The assigned effect-index.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_uint("effect-index",
-				 i18n_pspec("effect-index of the plugin"),
-				 i18n_pspec("The effect-index this plugin is assigned with"),
+				 i18n_pspec("effect index of the plugin"),
+				 i18n_pspec("The effect's index this plugin is assigned with"),
 				 0,
-				 G_MAXUINT,
+				 G_MAXUINT32,
 				 0,
 				 G_PARAM_READABLE | G_PARAM_WRITABLE);
   g_object_class_install_property(gobject,
 				  PROP_EFFECT_INDEX,
 				  param_spec);
 
+  /**
+   * AgsBasePlugin:plugin-so:
+   *
+   * The assigned plugin.so
+   * 
+   * Since: 2.0.0
+   */
+  param_spec = g_param_spec_pointer("plugin-so",
+				    i18n_pspec("plugin.so"),
+				    i18n_pspec("The plugin.so"),
+				    G_PARAM_READABLE | G_PARAM_WRITABLE);
+  g_object_class_install_property(gobject,
+				  PROP_PLUGIN_SO,
+				  param_spec);
+  
   /**
    * AgsBasePlugin:ui-filename:
    *
    * The assigned UI filename.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_string("ui-filename",
 				   i18n_pspec("UI filename of the plugin"),
@@ -214,7 +247,7 @@ ags_base_plugin_class_init(AgsBasePluginClass *base_plugin)
    *
    * The assigned ui-effect.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_string("ui-effect",
 				   i18n_pspec("UI effect of the plugin"),
@@ -230,25 +263,40 @@ ags_base_plugin_class_init(AgsBasePluginClass *base_plugin)
    *
    * The assigned ui-effect-index.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_uint("ui-effect-index",
 				 i18n_pspec("UI effect-index of the plugin"),
 				 i18n_pspec("The UI effect-index this plugin is assigned with"),
 				 0,
-				 G_MAXUINT,
+				 G_MAXUINT32,
 				 0,
 				 G_PARAM_READABLE | G_PARAM_WRITABLE);
   g_object_class_install_property(gobject,
 				  PROP_UI_EFFECT_INDEX,
 				  param_spec);
 
+  /**
+   * AgsBasePlugin:ui-plugin-so:
+   *
+   * The assigned ui_plugin.so
+   * 
+   * Since: 2.0.0
+   */
+  param_spec = g_param_spec_pointer("ui-plugin-so",
+				    i18n_pspec("ui_plugin.so"),
+				    i18n_pspec("The ui_plugin.so"),
+				    G_PARAM_READABLE | G_PARAM_WRITABLE);
+  g_object_class_install_property(gobject,
+				  PROP_UI_PLUGIN_SO,
+				  param_spec);
+  
   /**
    * AgsBasePlugin:ui-plugin:
    *
    * The assigned ui-plugin.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_object("ui-plugin",
 				   i18n_pspec("ui-plugin of the plugin"),
@@ -275,12 +323,13 @@ ags_base_plugin_class_init(AgsBasePluginClass *base_plugin)
    * AgsBasePlugin::instantiate:
    * @base_plugin: the plugin to instantiate
    * @samplerate: the samplerate
+   * @buffer_size: the fixed buffer size
    *
    * The ::instantiate signal creates a new instance of plugin.
    * 
    * Returns: the new plugin instance
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   base_plugin_signals[INSTANTIATE] =
     g_signal_new("instantiate",
@@ -288,17 +337,21 @@ ags_base_plugin_class_init(AgsBasePluginClass *base_plugin)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET (AgsBasePluginClass, instantiate),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_POINTER__UINT,
-		 G_TYPE_POINTER, 1,
+		 ags_cclosure_marshal_POINTER__UINT_UINT,
+		 G_TYPE_POINTER, 2,
+		 G_TYPE_UINT,
 		 G_TYPE_UINT);
 
   /**
    * AgsBasePlugin::connect-port:
    * @base_plugin: the plugin to connect-port
+   * @plugin_handle: the plugin handle
+   * @port_index: the port index
+   * @data_location: the data location
    *
    * The ::connect-port signal creates a new instance of plugin.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   base_plugin_signals[CONNECT_PORT] =
     g_signal_new("connect-port",
@@ -306,7 +359,7 @@ ags_base_plugin_class_init(AgsBasePluginClass *base_plugin)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET (AgsBasePluginClass, connect_port),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_VOID__POINTER_UINT_POINTER,
+		 ags_cclosure_marshal_VOID__POINTER_UINT_POINTER,
 		 G_TYPE_NONE, 3,
 		 G_TYPE_POINTER,
 		 G_TYPE_UINT,
@@ -315,10 +368,11 @@ ags_base_plugin_class_init(AgsBasePluginClass *base_plugin)
   /**
    * AgsBasePlugin::activate:
    * @base_plugin: the plugin to activate
+   * @plugin_handle: the plugin handle
    *
    * The ::activate signal creates a new instance of plugin.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   base_plugin_signals[ACTIVATE] =
     g_signal_new("activate",
@@ -333,10 +387,11 @@ ags_base_plugin_class_init(AgsBasePluginClass *base_plugin)
   /**
    * AgsBasePlugin::deactivate:
    * @base_plugin: the plugin to deactivate
+   * @plugin_handle: the plugin handle
    *
    * The ::deactivate signal creates a new instance of plugin.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   base_plugin_signals[DEACTIVATE] =
     g_signal_new("deactivate",
@@ -351,10 +406,13 @@ ags_base_plugin_class_init(AgsBasePluginClass *base_plugin)
   /**
    * AgsBasePlugin::run:
    * @base_plugin: the plugin to run
+   * @plugin_handle: the plugin handle
+   * @seq_event: the MIDI data
+   * @frame_count: the frame count
    *
    * The ::run signal creates a new instance of plugin.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   base_plugin_signals[RUN] =
     g_signal_new("run",
@@ -362,7 +420,7 @@ ags_base_plugin_class_init(AgsBasePluginClass *base_plugin)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET (AgsBasePluginClass, run),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_VOID__POINTER_POINTER_UINT,
+		 ags_cclosure_marshal_VOID__POINTER_POINTER_UINT,
 		 G_TYPE_NONE, 3,
 		 G_TYPE_POINTER,
 		 G_TYPE_POINTER,
@@ -374,7 +432,7 @@ ags_base_plugin_class_init(AgsBasePluginClass *base_plugin)
    *
    * The ::load_plugin signal creates a new instance of plugin.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   base_plugin_signals[LOAD_PLUGIN] =
     g_signal_new("load_plugin",
@@ -389,31 +447,37 @@ ags_base_plugin_class_init(AgsBasePluginClass *base_plugin)
 void
 ags_base_plugin_init(AgsBasePlugin *base_plugin)
 {
+  pthread_mutex_t *mutex;
+  pthread_mutexattr_t *attr;
+
   base_plugin->flags = 0;
 
-  /* create mutex */
-  base_plugin->obj_mutexattr = (pthread_mutexattr_t *) malloc(sizeof(pthread_mutexattr_t));
-  pthread_mutexattr_init(base_plugin->obj_mutexattr);
-  pthread_mutexattr_settype(base_plugin->obj_mutexattr,
+  /* add base plugin mutex */
+  base_plugin->obj_mutexattr = 
+    attr = (pthread_mutexattr_t *) malloc(sizeof(pthread_mutexattr_t));
+  pthread_mutexattr_init(attr);
+  pthread_mutexattr_settype(attr,
 			    PTHREAD_MUTEX_RECURSIVE);
 
 #ifdef __linux__
-  pthread_mutexattr_setprotocol(base_plugin->obj_mutexattr,
+  pthread_mutexattr_setprotocol(attr,
 				PTHREAD_PRIO_INHERIT);
 #endif
 
-  base_plugin->obj_mutex = (pthread_mutex_t *) malloc(sizeof(pthread_mutex_t));
-  pthread_mutex_init(base_plugin->obj_mutex,
-		     base_plugin->obj_mutexattr);
-  
-  base_plugin->id = NULL;
+  base_plugin->obj_mutex = 
+    mutex = (pthread_mutex_t *) malloc(sizeof(pthread_mutex_t));
+  pthread_mutex_init(mutex,
+		     attr);
+
+  /*  */
+  base_plugin->uuid = NULL;
   
   base_plugin->filename = NULL;
   base_plugin->effect = NULL;
 
   base_plugin->port_group_count = 0;
   base_plugin->port_group = NULL;
-  base_plugin->port = NULL;
+  base_plugin->plugin_port = NULL;
 
   base_plugin->effect_index = 0;
   base_plugin->plugin_so = NULL;
@@ -431,24 +495,25 @@ ags_base_plugin_set_property(GObject *gobject,
 {
   AgsBasePlugin *base_plugin;
 
+  pthread_mutex_t *base_plugin_mutex;
+
   base_plugin = AGS_BASE_PLUGIN(gobject);
 
+  /* get base plugin mutex */
+  pthread_mutex_lock(ags_base_plugin_get_class_mutex());
+  
+  base_plugin_mutex = base_plugin->obj_mutex;
+  
+  pthread_mutex_unlock(ags_base_plugin_get_class_mutex());
+
   switch(prop_id){
-  case PROP_ID:
+  case PROP_UUID:
     {
-      gchar *id;
+      pthread_mutex_lock(base_plugin_mutex);
 
-      id = (gchar *) g_value_get_string(value);
+      base_plugin->uuid = (AgsUUID *) g_value_get_pointer(value);
 
-      if(base_plugin->id == id){
-	return;
-      }
-      
-      if(base_plugin->id != NULL){
-	g_free(base_plugin->id);
-      }
-
-      base_plugin->id = g_strdup(id);
+      pthread_mutex_unlock(base_plugin_mutex);
     }
     break;
   case PROP_FILENAME:
@@ -457,7 +522,11 @@ ags_base_plugin_set_property(GObject *gobject,
 
       filename = (gchar *) g_value_get_string(value);
 
+      pthread_mutex_lock(base_plugin_mutex);
+
       if(base_plugin->filename == filename){
+	pthread_mutex_unlock(base_plugin_mutex);
+	
 	return;
       }
       
@@ -466,6 +535,8 @@ ags_base_plugin_set_property(GObject *gobject,
       }
 
       base_plugin->filename = g_strdup(filename);
+
+      pthread_mutex_unlock(base_plugin_mutex);
     }
     break;
   case PROP_EFFECT:
@@ -474,7 +545,11 @@ ags_base_plugin_set_property(GObject *gobject,
 
       effect = (gchar *) g_value_get_string(value);
 
+      pthread_mutex_lock(base_plugin_mutex);
+
       if(base_plugin->effect == effect){
+	pthread_mutex_unlock(base_plugin_mutex);
+
 	return;
       }
       
@@ -483,6 +558,30 @@ ags_base_plugin_set_property(GObject *gobject,
       }
 
       base_plugin->effect = g_strdup(effect);
+
+      pthread_mutex_unlock(base_plugin_mutex);
+    }
+    break;
+  case PROP_PLUGIN_PORT:
+    {
+      AgsPluginPort *plugin_port;
+
+      plugin_port = (AgsPluginPort *) g_value_get_pointer(value);
+
+      pthread_mutex_lock(base_plugin_mutex);
+
+      if(!AGS_IS_PLUGIN_PORT(plugin_port) ||
+	 g_list_find(base_plugin->plugin_port, plugin_port) != NULL){
+	pthread_mutex_unlock(base_plugin_mutex);
+	
+	return;
+      }
+
+      g_object_ref(plugin_port);
+      base_plugin->plugin_port = g_list_prepend(base_plugin->plugin_port,
+						plugin_port);
+
+      pthread_mutex_unlock(base_plugin_mutex);
     }
     break;
   case PROP_EFFECT_INDEX:
@@ -491,7 +590,20 @@ ags_base_plugin_set_property(GObject *gobject,
 
       effect_index = g_value_get_uint(value);
 
+      pthread_mutex_lock(base_plugin_mutex);
+
       base_plugin->effect_index = effect_index;
+
+      pthread_mutex_unlock(base_plugin_mutex);
+    }
+    break;
+  case PROP_PLUGIN_SO:
+    {
+      pthread_mutex_lock(base_plugin_mutex);
+
+      base_plugin->plugin_so = g_value_get_pointer(value);
+
+      pthread_mutex_unlock(base_plugin_mutex);
     }
     break;
   case PROP_UI_FILENAME:
@@ -500,7 +612,11 @@ ags_base_plugin_set_property(GObject *gobject,
 
       ui_filename = (gchar *) g_value_get_string(value);
 
+      pthread_mutex_lock(base_plugin_mutex);
+
       if(base_plugin->ui_filename == ui_filename){
+	pthread_mutex_unlock(base_plugin_mutex);
+
 	return;
       }
       
@@ -509,6 +625,8 @@ ags_base_plugin_set_property(GObject *gobject,
       }
 
       base_plugin->ui_filename = g_strdup(ui_filename);
+
+      pthread_mutex_unlock(base_plugin_mutex);
     }
     break;
   case PROP_UI_EFFECT:
@@ -517,7 +635,11 @@ ags_base_plugin_set_property(GObject *gobject,
 
       ui_effect = (gchar *) g_value_get_string(value);
 
+      pthread_mutex_lock(base_plugin_mutex);
+
       if(base_plugin->ui_effect == ui_effect){
+	pthread_mutex_unlock(base_plugin_mutex);
+
 	return;
       }
       
@@ -526,6 +648,8 @@ ags_base_plugin_set_property(GObject *gobject,
       }
 
       base_plugin->ui_effect = g_strdup(ui_effect);
+
+      pthread_mutex_unlock(base_plugin_mutex);
     }
     break;
   case PROP_UI_EFFECT_INDEX:
@@ -534,7 +658,20 @@ ags_base_plugin_set_property(GObject *gobject,
 
       ui_effect_index = g_value_get_uint(value);
 
+      pthread_mutex_lock(base_plugin_mutex);
+
       base_plugin->ui_effect_index = ui_effect_index;
+
+      pthread_mutex_unlock(base_plugin_mutex);
+    }
+    break;
+  case PROP_UI_PLUGIN_SO:
+    {
+      pthread_mutex_lock(base_plugin_mutex);
+
+      base_plugin->ui_plugin_so = g_value_get_pointer(value);
+
+      pthread_mutex_unlock(base_plugin_mutex);
     }
     break;
   case PROP_UI_PLUGIN:
@@ -543,7 +680,11 @@ ags_base_plugin_set_property(GObject *gobject,
 
       ui_plugin = g_value_get_object(value);
 
+      pthread_mutex_lock(base_plugin_mutex);
+
       if(base_plugin->ui_plugin == ui_plugin){
+	pthread_mutex_unlock(base_plugin_mutex);
+
 	return;
       }
 
@@ -556,6 +697,8 @@ ags_base_plugin_set_property(GObject *gobject,
       }
       
       base_plugin->ui_plugin = ui_plugin;
+
+      pthread_mutex_unlock(base_plugin_mutex);
     }
     break;
   default:
@@ -572,47 +715,115 @@ ags_base_plugin_get_property(GObject *gobject,
 {
   AgsBasePlugin *base_plugin;
 
+  pthread_mutex_t *base_plugin_mutex;
+
   base_plugin = AGS_BASE_PLUGIN(gobject);
 
+  /* get base plugin mutex */
+  pthread_mutex_lock(ags_base_plugin_get_class_mutex());
+  
+  base_plugin_mutex = base_plugin->obj_mutex;
+  
+  pthread_mutex_unlock(ags_base_plugin_get_class_mutex());
+
   switch(prop_id){
-  case PROP_ID:
+  case PROP_UUID:
     {
-      g_value_set_string(value, base_plugin->id);
+      pthread_mutex_lock(base_plugin_mutex);
+      
+      g_value_set_pointer(value, base_plugin->uuid);
+
+      pthread_mutex_unlock(base_plugin_mutex);
     }
     break;
   case PROP_FILENAME:
     {
+      pthread_mutex_lock(base_plugin_mutex);
+
       g_value_set_string(value, base_plugin->filename);
+
+      pthread_mutex_unlock(base_plugin_mutex);
     }
     break;
   case PROP_EFFECT:
     {
+      pthread_mutex_lock(base_plugin_mutex);
+
       g_value_set_string(value, base_plugin->effect);
+
+      pthread_mutex_unlock(base_plugin_mutex);
+    }
+    break;
+  case PROP_PLUGIN_PORT:
+    {
+      pthread_mutex_lock(base_plugin_mutex);
+      
+      g_value_set_pointer(value, g_list_copy(base_plugin->plugin_port));
+
+      pthread_mutex_unlock(base_plugin_mutex);
     }
     break;
   case PROP_EFFECT_INDEX:
     {
+      pthread_mutex_lock(base_plugin_mutex);
+
       g_value_set_uint(value, base_plugin->effect_index);
+
+      pthread_mutex_unlock(base_plugin_mutex);
+    }
+    break;
+  case PROP_PLUGIN_SO:
+    {
+      pthread_mutex_lock(base_plugin_mutex);
+      
+      g_value_set_pointer(value, base_plugin->plugin_so);
+
+      pthread_mutex_unlock(base_plugin_mutex);
     }
     break;
   case PROP_UI_FILENAME:
     {
+      pthread_mutex_lock(base_plugin_mutex);
+
       g_value_set_string(value, base_plugin->ui_filename);
+
+      pthread_mutex_unlock(base_plugin_mutex);
     }
     break;
   case PROP_UI_EFFECT:
     {
+      pthread_mutex_lock(base_plugin_mutex);
+
       g_value_set_string(value, base_plugin->ui_effect);
+
+      pthread_mutex_unlock(base_plugin_mutex);
     }
     break;
   case PROP_UI_EFFECT_INDEX:
     {
+      pthread_mutex_lock(base_plugin_mutex);
+
       g_value_set_uint(value, base_plugin->ui_effect_index);
+
+      pthread_mutex_unlock(base_plugin_mutex);
+    }
+    break;
+  case PROP_UI_PLUGIN_SO:
+    {
+      pthread_mutex_lock(base_plugin_mutex);
+      
+      g_value_set_pointer(value, base_plugin->ui_plugin_so);
+
+      pthread_mutex_unlock(base_plugin_mutex);
     }
     break;
   case PROP_UI_PLUGIN:
     {
+      pthread_mutex_lock(base_plugin_mutex);
+
       g_value_set_object(value, base_plugin->ui_plugin);
+
+      pthread_mutex_unlock(base_plugin_mutex);
     }
     break;
   default:
@@ -634,6 +845,14 @@ ags_base_plugin_dispose(GObject *gobject)
     base_plugin->ui_plugin = NULL;
   }
 
+  /* plugin port */
+  if(base_plugin->plugin_port != NULL){
+    g_list_free_full(base_plugin->plugin_port,
+		     g_object_unref);
+
+    base_plugin->plugin_port = NULL;
+  }
+
   /* call parent */
   G_OBJECT_CLASS(ags_base_plugin_parent_class)->dispose(gobject);
 }
@@ -645,23 +864,46 @@ ags_base_plugin_finalize(GObject *gobject)
 
   base_plugin = AGS_BASE_PLUGIN(gobject);
 
+  /* destroy object mutex */
   pthread_mutex_destroy(base_plugin->obj_mutex);
   free(base_plugin->obj_mutex);
 
   pthread_mutexattr_destroy(base_plugin->obj_mutexattr);
   free(base_plugin->obj_mutexattr);
 
+  /* uuid */
+  ags_uuid_free(base_plugin->uuid);
+  
+  /* filename and effect */
   g_free(base_plugin->filename);
   g_free(base_plugin->effect);
 
+  /* UI filename and effect */
+  g_free(base_plugin->ui_filename);
+  g_free(base_plugin->ui_effect);
+
+  /* plugin port */
+  g_list_free_full(base_plugin->plugin_port,
+		   g_object_unref);
+
+  /* ui-plugin */
   if(base_plugin->ui_plugin != NULL){
     g_object_unref(base_plugin->ui_plugin);
   }
-
+  
   /* call parent */
   G_OBJECT_CLASS(ags_base_plugin_parent_class)->finalize(gobject);
 }
 
+/**
+ * ags_base_plugin_get_class_mutex:
+ * 
+ * Use this function's returned mutex to access mutex fields.
+ *
+ * Returns: the class mutex
+ * 
+ * Since: 2.0.0
+ */
 pthread_mutex_t*
 ags_base_plugin_get_class_mutex()
 {
@@ -669,105 +911,108 @@ ags_base_plugin_get_class_mutex()
 }
 
 /**
- * ags_port_descriptor_alloc:
+ * ags_base_plugin_test_flags:
+ * @base_plugin: the #AgsBasePlugin
+ * @flags: the flags
  * 
- * Alloc the #AgsPortDescriptor-struct
- *
- * Returns: the #AgsPortDescriptor-struct
- *
- * Since: 1.0.0
+ * Test @flags to be set on @recall.
+ * 
+ * Returns: %TRUE if flags are set, else %FALSE
+ * 
+ * Since: 2.0.0
  */
-AgsPortDescriptor*
-ags_port_descriptor_alloc()
+gboolean
+ags_base_plugin_test_flags(AgsBasePlugin *base_plugin, guint flags)
 {
-  AgsPortDescriptor *port_descriptor;
-
-  port_descriptor = (AgsPortDescriptor *) malloc(sizeof(AgsPortDescriptor));
-
-  port_descriptor->flags = 0;
-
-  port_descriptor->port_index = 0;
-
-  port_descriptor->port_name = NULL;
-  port_descriptor->port_symbol = NULL;
-
-  port_descriptor->scale_steps = -1;
-  port_descriptor->scale_points = NULL;
-  port_descriptor->scale_value = NULL;
+  gboolean retval;
   
-  port_descriptor->lower_value = g_new0(GValue,
-					1);
-  port_descriptor->upper_value = g_new0(GValue,
-					1);
+  pthread_mutex_t *base_plugin_mutex;
 
-  port_descriptor->default_value = g_new0(GValue,
-					  1);
-
-  port_descriptor->user_data = NULL;
+  if(!AGS_IS_BASE_PLUGIN(base_plugin)){
+    return(FALSE);
+  }
   
-  return(port_descriptor);
+  /* get base plugin mutex */
+  pthread_mutex_lock(ags_base_plugin_get_class_mutex());
+  
+  base_plugin_mutex = base_plugin->obj_mutex;
+  
+  pthread_mutex_unlock(ags_base_plugin_get_class_mutex());
+
+  /* test flags */
+  pthread_mutex_lock(base_plugin_mutex);
+
+  retval = ((flags & (base_plugin->flags)) != 0) ? TRUE: FALSE;
+  
+  pthread_mutex_unlock(base_plugin_mutex);
+
+  return(retval);
 }
 
 /**
- * ags_port_descriptor_free:
- * @port_descriptor: the #AgsPortDescriptor-struct
- * 
- * Free the #AgsPortDescriptor-struct
+ * ags_base_plugin_set_flags:
+ * @base_plugin: the #AgsBasePlugin
+ * @flags: the flags
  *
- * Since: 1.0.0
+ * Set flags.
+ * 
+ * Since: 2.0.0
  */
 void
-ags_port_descriptor_free(AgsPortDescriptor *port_descriptor)
+ags_base_plugin_set_flags(AgsBasePlugin *base_plugin, guint flags)
 {
-  if(port_descriptor->port_name != NULL){
-    g_free(port_descriptor->port_name);
+  pthread_mutex_t *base_plugin_mutex;
+
+  if(!AGS_IS_BASE_PLUGIN(base_plugin)){
+    return;
   }
   
-  if(port_descriptor->port_symbol != NULL){
-    g_free(port_descriptor->port_symbol);
-  }
+  /* get base plugin mutex */
+  pthread_mutex_lock(ags_base_plugin_get_class_mutex());
+  
+  base_plugin_mutex = base_plugin->obj_mutex;
+  
+  pthread_mutex_unlock(ags_base_plugin_get_class_mutex());
 
-  if(port_descriptor->scale_points != NULL){
-    g_free(port_descriptor->scale_points);
-  }
+  /* set flags */
+  pthread_mutex_lock(base_plugin_mutex);
 
-  g_free(port_descriptor->lower_value);
-  g_free(port_descriptor->upper_value);
-
-  g_free(port_descriptor->default_value);
-
-  free(port_descriptor);
+  base_plugin->flags |= flags;
+  
+  pthread_mutex_unlock(base_plugin_mutex);
 }
 
 /**
- * ags_port_descriptor_find_symbol:
- * @port_descriptor: the #GList-struct containing #AgsPortDescriptor
- * @port_symbol: the port symbol
+ * ags_base_plugin_unset_flags:
+ * @base_plugin: the #AgsBasePlugin
+ * @flags: the flags
+ *
+ * Unset flags.
  * 
- * Find @port_symbol within @port_descriptor.
- * 
- * Returns: the matching #GList-struct containing #AgsPortDescriptor
- * 
- * Since: 1.0.0.8 
+ * Since: 2.0.0
  */
-GList*
-ags_port_descriptor_find_symbol(GList *port_descriptor,
-				gchar *port_symbol)
+void
+ags_base_plugin_unset_flags(AgsBasePlugin *base_plugin, guint flags)
 {
-  if(port_symbol == NULL){
-    return(NULL);
-  }
+  pthread_mutex_t *base_plugin_mutex;
 
-  while(port_descriptor != NULL){
-    if(!g_strcmp0(port_symbol,
-		  AGS_PORT_DESCRIPTOR(port_descriptor->data)->port_symbol)){
-      return(port_descriptor);
-    }
-    
-    port_descriptor = port_descriptor->next;
+  if(!AGS_IS_BASE_PLUGIN(base_plugin)){
+    return;
   }
   
-  return(NULL);
+  /* get base plugin mutex */
+  pthread_mutex_lock(ags_base_plugin_get_class_mutex());
+  
+  base_plugin_mutex = base_plugin->obj_mutex;
+  
+  pthread_mutex_unlock(ags_base_plugin_get_class_mutex());
+
+  /* unset flags */
+  pthread_mutex_lock(base_plugin_mutex);
+
+  base_plugin->flags &= (~flags);
+  
+  pthread_mutex_unlock(base_plugin_mutex);
 }
 
 /**
@@ -779,14 +1024,36 @@ ags_port_descriptor_find_symbol(GList *port_descriptor,
  *
  * Returns: the next matching #GList-struct
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 GList*
 ags_base_plugin_find_filename(GList *base_plugin, gchar *filename)
 {
+  AgsBasePlugin *current_base_plugin;
+  
+  gboolean success;
+  
+  pthread_mutex_t *base_plugin_mutex;
+
   while(base_plugin != NULL){
-    if(!g_ascii_strcasecmp(AGS_BASE_PLUGIN(base_plugin->data)->filename,
-			   filename)){
+    current_base_plugin = AGS_BASE_PLUGIN(base_plugin->data);
+    
+    /* get base plugin mutex */
+    pthread_mutex_lock(ags_base_plugin_get_class_mutex());
+  
+    base_plugin_mutex = current_base_plugin->obj_mutex;
+  
+    pthread_mutex_unlock(ags_base_plugin_get_class_mutex());
+
+    /* check filename */
+    pthread_mutex_lock(base_plugin_mutex);
+
+    success = (!g_ascii_strcasecmp(current_base_plugin->filename,
+				   filename)) ? TRUE: FALSE;
+    
+    pthread_mutex_unlock(base_plugin_mutex);
+
+    if(success){
       return(base_plugin);
     }
 
@@ -806,16 +1073,38 @@ ags_base_plugin_find_filename(GList *base_plugin, gchar *filename)
  *
  * Returns: the next matching #GList-struct
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 GList*
 ags_base_plugin_find_effect(GList *base_plugin, gchar *filename, gchar *effect)
 {
+  AgsBasePlugin *current_base_plugin;
+  
+  gboolean success;
+  
+  pthread_mutex_t *base_plugin_mutex;
+
   while(base_plugin != NULL){
-    if(!g_ascii_strcasecmp(AGS_BASE_PLUGIN(base_plugin->data)->filename,
-			   filename) &&
-       !g_ascii_strcasecmp(AGS_BASE_PLUGIN(base_plugin->data)->effect,
-			   effect)){
+    current_base_plugin = AGS_BASE_PLUGIN(base_plugin->data);
+    
+    /* get base plugin mutex */
+    pthread_mutex_lock(ags_base_plugin_get_class_mutex());
+  
+    base_plugin_mutex = current_base_plugin->obj_mutex;
+  
+    pthread_mutex_unlock(ags_base_plugin_get_class_mutex());
+
+    /* check filename and effect*/
+    pthread_mutex_lock(base_plugin_mutex);
+
+    success = (!g_ascii_strcasecmp(current_base_plugin->filename,
+				   filename) &&
+	       !g_ascii_strcasecmp(current_base_plugin->effect,
+				   effect)) ? TRUE: FALSE;
+    
+    pthread_mutex_unlock(base_plugin_mutex);
+
+    if(success){
       return(base_plugin);
     }
 
@@ -835,15 +1124,37 @@ ags_base_plugin_find_effect(GList *base_plugin, gchar *filename, gchar *effect)
  *
  * Returns: the next matching #GList-struct
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 GList*
 ags_base_plugin_find_ui_effect_index(GList *base_plugin, gchar *ui_filename, guint ui_effect_index)
 {
+  AgsBasePlugin *current_base_plugin;
+  
+  gboolean success;
+  
+  pthread_mutex_t *base_plugin_mutex;
+
   while(base_plugin != NULL){
-    if(!g_ascii_strcasecmp(AGS_BASE_PLUGIN(base_plugin->data)->ui_filename,
-			   ui_filename) &&
-       AGS_BASE_PLUGIN(base_plugin->data)->ui_effect_index == ui_effect_index){
+    current_base_plugin = AGS_BASE_PLUGIN(base_plugin->data);
+    
+    /* get base plugin mutex */
+    pthread_mutex_lock(ags_base_plugin_get_class_mutex());
+  
+    base_plugin_mutex = current_base_plugin->obj_mutex;
+  
+    pthread_mutex_unlock(ags_base_plugin_get_class_mutex());
+
+    /* check UI filename and effect index */
+    pthread_mutex_lock(base_plugin_mutex);
+
+    success = (!g_ascii_strcasecmp(current_base_plugin->ui_filename,
+				   ui_filename) &&
+	       current_base_plugin->ui_effect_index == ui_effect_index) ? TRUE: FALSE;
+    
+    pthread_mutex_unlock(base_plugin_mutex);
+
+    if(success){
       return(base_plugin);
     }
 
@@ -861,7 +1172,7 @@ ags_base_plugin_find_ui_effect_index(GList *base_plugin, gchar *ui_filename, gui
  * 
  * Returns: the sorted #GList-struct
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 GList*
 ags_base_plugin_sort(GList *base_plugin)
@@ -871,8 +1182,51 @@ ags_base_plugin_sort(GList *base_plugin)
   auto gint ags_base_plugin_sort_compare_function(gpointer a, gpointer b);
 
   gint ags_base_plugin_sort_compare_function(gpointer a, gpointer b){
-    return(strcmp(AGS_BASE_PLUGIN(a)->effect,
-		  AGS_BASE_PLUGIN(b)->effect));
+    AgsBasePlugin *a_plugin;
+    AgsBasePlugin *b_plugin;
+    
+    gchar *a_effect;
+    gchar *b_effect;
+
+    gint retval;
+    
+    pthread_mutex_t *a_plugin_mutex;
+    pthread_mutex_t *b_plugin_mutex;
+
+    /* a and b */
+    a_plugin = AGS_BASE_PLUGIN(a);
+    b_plugin = AGS_BASE_PLUGIN(b);
+    
+    /* get base plugin mutex - a and b */
+    pthread_mutex_lock(ags_base_plugin_get_class_mutex());
+  
+    a_plugin_mutex = a_plugin->obj_mutex;
+    b_plugin_mutex = b_plugin->obj_mutex;
+  
+    pthread_mutex_unlock(ags_base_plugin_get_class_mutex());
+
+    /* duplicate effect - a */
+    pthread_mutex_lock(a_plugin_mutex);
+
+    a_effect = g_strdup(a_plugin->effect);
+    
+    pthread_mutex_unlock(a_plugin_mutex);
+
+    /* duplicate effect - b */
+    pthread_mutex_lock(b_plugin_mutex);
+
+    b_effect = g_strdup(b_plugin->effect);
+    
+    pthread_mutex_unlock(b_plugin_mutex);
+
+    /* compare and free */
+    retval = strcmp(a_effect,
+		    b_effect);
+
+    g_free(a_effect);
+    g_free(b_effect);
+    
+    return(retval);
   }
 
   if(base_plugin == NULL){
@@ -902,16 +1256,17 @@ ags_base_plugin_apply_port_group_by_prefix(AgsBasePlugin *base_plugin)
  * ags_base_plugin_instantiate:
  * @base_plugin: the #AgsBasePlugin
  * @samplerate: the samplerate
+ * @buffer_size: the fixed buffer size
  *
  * Instantiate the plugin
  *
  * Returns: the new plugin instance handle
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 gpointer
 ags_base_plugin_instantiate(AgsBasePlugin *base_plugin,
-			    guint samplerate)
+			    guint samplerate, guint buffer_size)
 {
   gpointer retval;
   
@@ -920,7 +1275,7 @@ ags_base_plugin_instantiate(AgsBasePlugin *base_plugin,
   g_object_ref(G_OBJECT(base_plugin));
   g_signal_emit(G_OBJECT(base_plugin),
 		base_plugin_signals[INSTANTIATE], 0,
-		samplerate,
+		samplerate, buffer_size,
 		&retval);
   g_object_unref(G_OBJECT(base_plugin));
 
@@ -936,7 +1291,7 @@ ags_base_plugin_instantiate(AgsBasePlugin *base_plugin,
  *
  * Connect a plugin instance.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_base_plugin_connect_port(AgsBasePlugin *base_plugin, gpointer plugin_handle, guint port_index, gpointer data_location)
@@ -956,7 +1311,7 @@ ags_base_plugin_connect_port(AgsBasePlugin *base_plugin, gpointer plugin_handle,
  *
  * Activate a plugin instance
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_base_plugin_activate(AgsBasePlugin *base_plugin, gpointer plugin_handle)
@@ -976,7 +1331,7 @@ ags_base_plugin_activate(AgsBasePlugin *base_plugin, gpointer plugin_handle)
  *
  * Deactivat a plugin instance
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_base_plugin_deactivate(AgsBasePlugin *base_plugin, gpointer plugin_handle)
@@ -998,7 +1353,7 @@ ags_base_plugin_deactivate(AgsBasePlugin *base_plugin, gpointer plugin_handle)
  *
  * Deactivat a plugin instance
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_base_plugin_run(AgsBasePlugin *base_plugin,
@@ -1023,7 +1378,7 @@ ags_base_plugin_run(AgsBasePlugin *base_plugin,
  *
  * Load the plugin
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_base_plugin_load_plugin(AgsBasePlugin *base_plugin)
@@ -1046,7 +1401,7 @@ ags_base_plugin_load_plugin(AgsBasePlugin *base_plugin)
  *
  * Returns: a new #AgsBasePlugin
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 AgsBasePlugin*
 ags_base_plugin_new(gchar *filename, gchar *effect, guint effect_index)
